@@ -22,6 +22,7 @@ MongoDB.
 from __future__ import annotations
 
 import datetime
+import logging
 import random
 import struct
 from io import BytesIO as _BytesIO
@@ -29,16 +30,12 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
     Iterable,
-    List,
     Mapping,
     MutableMapping,
     NoReturn,
     Optional,
-    Tuple,
     Union,
-    cast,
 )
 
 import bson
@@ -50,7 +47,6 @@ from bson.raw_bson import (
     RawBSONDocument,
     _inflate_bson,
 )
-from bson.son import SON
 
 try:
     from pymongo import _cmessage  # type: ignore[attr-defined]
@@ -70,6 +66,7 @@ from pymongo.errors import (
 )
 from pymongo.hello import HelloCompat
 from pymongo.helpers import _handle_reauth
+from pymongo.logger import _COMMAND_LOGGER, _CommandStatusMessage, _debug_log
 from pymongo.read_preferences import ReadPreference
 from pymongo.write_concern import WriteConcern
 
@@ -109,14 +106,14 @@ _OP_MAP = {
 }
 _FIELD_MAP = {"insert": "documents", "update": "updates", "delete": "deletes"}
 
-_UNICODE_REPLACE_CODEC_OPTIONS: "CodecOptions[Mapping[str, Any]]" = CodecOptions(
+_UNICODE_REPLACE_CODEC_OPTIONS: CodecOptions[Mapping[str, Any]] = CodecOptions(
     unicode_decode_error_handler="replace"
 )
 
 
 def _randint() -> int:
     """Generate a pseudo random 32 bit integer."""
-    return random.randint(MIN_INT32, MAX_INT32)
+    return random.randint(MIN_INT32, MAX_INT32)  # noqa: S311
 
 
 def _maybe_add_read_preference(
@@ -132,19 +129,19 @@ def _maybe_add_read_preference(
     # the secondaryOkay bit has the same effect).
     if mode and (mode != ReadPreference.SECONDARY_PREFERRED.mode or len(document) > 1):
         if "$query" not in spec:
-            spec = SON([("$query", spec)])
+            spec = {"$query": spec}
         spec["$readPreference"] = document
     return spec
 
 
-def _convert_exception(exception: Exception) -> Dict[str, Any]:
+def _convert_exception(exception: Exception) -> dict[str, Any]:
     """Convert an Exception into a failure document for publishing."""
     return {"errmsg": str(exception), "errtype": exception.__class__.__name__}
 
 
 def _convert_write_result(
     operation: str, command: Mapping[str, Any], result: Mapping[str, Any]
-) -> Dict[str, Any]:
+) -> dict[str, Any]:
     """Convert a legacy write result to write command format."""
     # Based on _merge_legacy from bulk.py
     affected = result.get("n", 0)
@@ -178,33 +175,29 @@ def _convert_write_result(
     return res
 
 
-_OPTIONS = SON(
-    [
-        ("tailable", 2),
-        ("oplogReplay", 8),
-        ("noCursorTimeout", 16),
-        ("awaitData", 32),
-        ("allowPartialResults", 128),
-    ]
-)
+_OPTIONS = {
+    "tailable": 2,
+    "oplogReplay": 8,
+    "noCursorTimeout": 16,
+    "awaitData": 32,
+    "allowPartialResults": 128,
+}
 
 
-_MODIFIERS = SON(
-    [
-        ("$query", "filter"),
-        ("$orderby", "sort"),
-        ("$hint", "hint"),
-        ("$comment", "comment"),
-        ("$maxScan", "maxScan"),
-        ("$maxTimeMS", "maxTimeMS"),
-        ("$max", "max"),
-        ("$min", "min"),
-        ("$returnKey", "returnKey"),
-        ("$showRecordId", "showRecordId"),
-        ("$showDiskLoc", "showRecordId"),  # <= MongoDb 3.0
-        ("$snapshot", "snapshot"),
-    ]
-)
+_MODIFIERS = {
+    "$query": "filter",
+    "$orderby": "sort",
+    "$hint": "hint",
+    "$comment": "comment",
+    "$maxScan": "maxScan",
+    "$maxTimeMS": "maxTimeMS",
+    "$max": "max",
+    "$min": "min",
+    "$returnKey": "returnKey",
+    "$showRecordId": "showRecordId",
+    "$showDiskLoc": "showRecordId",  # <= MongoDb 3.0
+    "$snapshot": "snapshot",
+}
 
 
 def _gen_find_command(
@@ -219,9 +212,9 @@ def _gen_find_command(
     collation: Optional[Mapping[str, Any]] = None,
     session: Optional[ClientSession] = None,
     allow_disk_use: Optional[bool] = None,
-) -> SON[str, Any]:
+) -> dict[str, Any]:
     """Generate a find command document."""
-    cmd: SON[str, Any] = SON([("find", coll)])
+    cmd: dict[str, Any] = {"find": coll}
     if "$query" in spec:
         cmd.update(
             [
@@ -265,9 +258,9 @@ def _gen_get_more_command(
     max_await_time_ms: Optional[int],
     comment: Optional[Any],
     conn: Connection,
-) -> SON[str, Any]:
+) -> dict[str, Any]:
     """Generate a getMore command document."""
-    cmd: SON[str, Any] = SON([("getMore", cursor_id), ("collection", coll)])
+    cmd: dict[str, Any] = {"getMore": cursor_id, "collection": coll}
     if batch_size:
         cmd["batchSize"] = batch_size
     if max_await_time_ms is not None:
@@ -340,7 +333,7 @@ class _Query:
         self.client = client
         self.allow_disk_use = allow_disk_use
         self.name = "find"
-        self._as_command: Optional[Tuple[SON[str, Any], str]] = None
+        self._as_command: Optional[tuple[dict[str, Any], str]] = None
         self.exhaust = exhaust
 
     def reset(self) -> None:
@@ -367,7 +360,7 @@ class _Query:
 
     def as_command(
         self, conn: Connection, apply_timeout: bool = False
-    ) -> Tuple[SON[str, Any], str]:
+    ) -> tuple[dict[str, Any], str]:
         """Return a find command document for this query."""
         # We use the command twice: on the wire and for command monitoring.
         # Generate it once, for speed and to avoid repeating side-effects.
@@ -375,7 +368,7 @@ class _Query:
             return self._as_command
 
         explain = "$explain" in self.spec
-        cmd: SON[str, Any] = _gen_find_command(
+        cmd: dict[str, Any] = _gen_find_command(
             self.coll,
             self.spec,
             self.fields,
@@ -390,7 +383,7 @@ class _Query:
         )
         if explain:
             self.name = "explain"
-            cmd = SON([("explain", cmd)])
+            cmd = {"explain": cmd}
         session = self.session
         conn.add_server_api(cmd)
         if session:
@@ -402,7 +395,7 @@ class _Query:
         # Support auto encryption
         client = self.client
         if client._encrypter and not client._encrypter._bypass_auto_encryption:
-            cmd = cast(SON[str, Any], client._encrypter.encrypt(self.db, cmd, self.codec_options))
+            cmd = client._encrypter.encrypt(self.db, cmd, self.codec_options)
         # Support CSOT
         if apply_timeout:
             conn.apply_timeout(client, cmd)
@@ -411,7 +404,7 @@ class _Query:
 
     def get_message(
         self, read_preference: _ServerMode, conn: Connection, use_cmd: bool = False
-    ) -> Tuple[int, bytes, int]:
+    ) -> tuple[int, bytes, int]:
         """Get a query message, possibly setting the secondaryOk bit."""
         # Use the read_preference decided by _socket_from_server.
         self.read_preference = read_preference
@@ -508,7 +501,7 @@ class _GetMore:
         self.client = client
         self.max_await_time_ms = max_await_time_ms
         self.conn_mgr = conn_mgr
-        self._as_command: Optional[Tuple[SON[str, Any], str]] = None
+        self._as_command: Optional[tuple[dict[str, Any], str]] = None
         self.exhaust = exhaust
         self.comment = comment
 
@@ -531,13 +524,13 @@ class _GetMore:
 
     def as_command(
         self, conn: Connection, apply_timeout: bool = False
-    ) -> Tuple[SON[str, Any], str]:
+    ) -> tuple[dict[str, Any], str]:
         """Return a getMore command document for this query."""
         # See _Query.as_command for an explanation of this caching.
         if self._as_command is not None:
             return self._as_command
 
-        cmd: SON[str, Any] = _gen_get_more_command(
+        cmd: dict[str, Any] = _gen_get_more_command(
             self.cursor_id,
             self.coll,
             self.ntoreturn,
@@ -552,7 +545,7 @@ class _GetMore:
         # Support auto encryption
         client = self.client
         if client._encrypter and not client._encrypter._bypass_auto_encryption:
-            cmd = cast(SON[str, Any], client._encrypter.encrypt(self.db, cmd, self.codec_options))
+            cmd = client._encrypter.encrypt(self.db, cmd, self.codec_options)
         # Support CSOT
         if apply_timeout:
             conn.apply_timeout(client, cmd=None)
@@ -561,14 +554,14 @@ class _GetMore:
 
     def get_message(
         self, dummy0: Any, conn: Connection, use_cmd: bool = False
-    ) -> Union[Tuple[int, bytes, int], Tuple[int, bytes]]:
+    ) -> Union[tuple[int, bytes, int], tuple[int, bytes]]:
         """Get a getmore message."""
         ns = self.namespace()
         ctx = conn.compression_context
 
         if use_cmd:
             spec = self.as_command(conn, apply_timeout=True)[0]
-            if self.conn_mgr:
+            if self.conn_mgr and self.exhaust:
                 flags = _OpMsg.EXHAUST_ALLOWED
             else:
                 flags = 0
@@ -639,7 +632,7 @@ _COMPRESSION_HEADER_SIZE = 25
 
 def _compress(
     operation: int, data: bytes, ctx: Union[SnappyContext, ZlibContext, ZstdContext]
-) -> Tuple[int, bytes]:
+) -> tuple[int, bytes]:
     """Takes message data, compresses it, and adds an OP_COMPRESSED header."""
     compressed = ctx.compress(data)
     request_id = _randint()
@@ -659,7 +652,7 @@ def _compress(
 _pack_header = struct.Struct("<iiii").pack
 
 
-def __pack_message(operation: int, data: bytes) -> Tuple[int, bytes]:
+def __pack_message(operation: int, data: bytes) -> tuple[int, bytes]:
     """Takes message data and adds a message header based on the operation.
 
     Returns the resultant message string.
@@ -678,9 +671,9 @@ def _op_msg_no_header(
     flags: int,
     command: Mapping[str, Any],
     identifier: str,
-    docs: Optional[List[Mapping[str, Any]]],
+    docs: Optional[list[Mapping[str, Any]]],
     opts: CodecOptions,
-) -> Tuple[bytes, int, int]:
+) -> tuple[bytes, int, int]:
     """Get a OP_MSG message.
 
     Note: this method handles multiple documents in a type one payload but
@@ -710,10 +703,10 @@ def _op_msg_compressed(
     flags: int,
     command: Mapping[str, Any],
     identifier: str,
-    docs: Optional[List[Mapping[str, Any]]],
+    docs: Optional[list[Mapping[str, Any]]],
     opts: CodecOptions,
     ctx: Union[SnappyContext, ZlibContext, ZstdContext],
-) -> Tuple[int, bytes, int, int]:
+) -> tuple[int, bytes, int, int]:
     """Internal OP_MSG message helper."""
     msg, total_size, max_bson_size = _op_msg_no_header(flags, command, identifier, docs, opts)
     rid, msg = _compress(2013, msg, ctx)
@@ -724,9 +717,9 @@ def _op_msg_uncompressed(
     flags: int,
     command: Mapping[str, Any],
     identifier: str,
-    docs: Optional[List[Mapping[str, Any]]],
+    docs: Optional[list[Mapping[str, Any]]],
     opts: CodecOptions,
-) -> Tuple[int, bytes, int, int]:
+) -> tuple[int, bytes, int, int]:
     """Internal compressed OP_MSG message helper."""
     data, total_size, max_bson_size = _op_msg_no_header(flags, command, identifier, docs, opts)
     request_id, op_message = __pack_message(2013, data)
@@ -734,7 +727,7 @@ def _op_msg_uncompressed(
 
 
 if _use_c:
-    _op_msg_uncompressed = _cmessage._op_msg  # noqa: F811
+    _op_msg_uncompressed = _cmessage._op_msg
 
 
 def _op_msg(
@@ -744,7 +737,7 @@ def _op_msg(
     read_preference: Optional[_ServerMode],
     opts: CodecOptions,
     ctx: Union[SnappyContext, ZlibContext, ZstdContext, None] = None,
-) -> Tuple[int, bytes, int, int]:
+) -> tuple[int, bytes, int, int]:
     """Get a OP_MSG message."""
     command["$db"] = dbname
     # getMore commands do not send $readPreference.
@@ -777,7 +770,7 @@ def _query_impl(
     query: Mapping[str, Any],
     field_selector: Optional[Mapping[str, Any]],
     opts: CodecOptions,
-) -> Tuple[bytes, int]:
+) -> tuple[bytes, int]:
     """Get an OP_QUERY message."""
     encoded = _dict_to_bson(query, False, opts)
     if field_selector:
@@ -809,7 +802,7 @@ def _query_compressed(
     field_selector: Optional[Mapping[str, Any]],
     opts: CodecOptions,
     ctx: Union[SnappyContext, ZlibContext, ZstdContext],
-) -> Tuple[int, bytes, int]:
+) -> tuple[int, bytes, int]:
     """Internal compressed query message helper."""
     op_query, max_bson_size = _query_impl(
         options, collection_name, num_to_skip, num_to_return, query, field_selector, opts
@@ -826,7 +819,7 @@ def _query_uncompressed(
     query: Mapping[str, Any],
     field_selector: Optional[Mapping[str, Any]],
     opts: CodecOptions,
-) -> Tuple[int, bytes, int]:
+) -> tuple[int, bytes, int]:
     """Internal query message helper."""
     op_query, max_bson_size = _query_impl(
         options, collection_name, num_to_skip, num_to_return, query, field_selector, opts
@@ -836,7 +829,7 @@ def _query_uncompressed(
 
 
 if _use_c:
-    _query_uncompressed = _cmessage._query_message  # noqa: F811
+    _query_uncompressed = _cmessage._query_message
 
 
 def _query(
@@ -848,7 +841,7 @@ def _query(
     field_selector: Optional[Mapping[str, Any]],
     opts: CodecOptions,
     ctx: Union[SnappyContext, ZlibContext, ZstdContext, None] = None,
-) -> Tuple[int, bytes, int]:
+) -> tuple[int, bytes, int]:
     """Get a **query** message."""
     if ctx:
         return _query_compressed(
@@ -879,20 +872,20 @@ def _get_more_compressed(
     num_to_return: int,
     cursor_id: int,
     ctx: Union[SnappyContext, ZlibContext, ZstdContext],
-) -> Tuple[int, bytes]:
+) -> tuple[int, bytes]:
     """Internal compressed getMore message helper."""
     return _compress(2005, _get_more_impl(collection_name, num_to_return, cursor_id), ctx)
 
 
 def _get_more_uncompressed(
     collection_name: str, num_to_return: int, cursor_id: int
-) -> Tuple[int, bytes]:
+) -> tuple[int, bytes]:
     """Internal getMore message helper."""
     return __pack_message(2005, _get_more_impl(collection_name, num_to_return, cursor_id))
 
 
 if _use_c:
-    _get_more_uncompressed = _cmessage._get_more_message  # noqa: F811
+    _get_more_uncompressed = _cmessage._get_more_message
 
 
 def _get_more(
@@ -900,7 +893,7 @@ def _get_more(
     num_to_return: int,
     cursor_id: int,
     ctx: Union[SnappyContext, ZlibContext, ZstdContext, None] = None,
-) -> Tuple[int, bytes]:
+) -> tuple[int, bytes]:
     """Get a **getMore** message."""
     if ctx:
         return _get_more_compressed(collection_name, num_to_return, cursor_id, ctx)
@@ -943,15 +936,15 @@ class _BulkWriteContext:
         self.publish = listeners.enabled_for_commands
         self.name = cmd_name
         self.field = _FIELD_MAP[self.name]
-        self.start_time = datetime.datetime.now() if self.publish else None
+        self.start_time = datetime.datetime.now()
         self.session = session
-        self.compress = True if conn.compression_context else False
+        self.compress = bool(conn.compression_context)
         self.op_type = op_type
         self.codec = codec
 
     def __batch_command(
-        self, cmd: MutableMapping[str, Any], docs: List[Mapping[str, Any]]
-    ) -> Tuple[int, bytes, List[Mapping[str, Any]]]:
+        self, cmd: MutableMapping[str, Any], docs: list[Mapping[str, Any]]
+    ) -> tuple[int, bytes, list[Mapping[str, Any]]]:
         namespace = self.db_name + ".$cmd"
         request_id, msg, to_send = _do_batched_op_msg(
             namespace, self.op_type, cmd, docs, self.codec, self
@@ -961,23 +954,23 @@ class _BulkWriteContext:
         return request_id, msg, to_send
 
     def execute(
-        self, cmd: MutableMapping[str, Any], docs: List[Mapping[str, Any]], client: MongoClient
-    ) -> Tuple[Mapping[str, Any], List[Mapping[str, Any]]]:
+        self, cmd: MutableMapping[str, Any], docs: list[Mapping[str, Any]], client: MongoClient
+    ) -> tuple[Mapping[str, Any], list[Mapping[str, Any]]]:
         request_id, msg, to_send = self.__batch_command(cmd, docs)
-        result = self.write_command(cmd, request_id, msg, to_send)
+        result = self.write_command(cmd, request_id, msg, to_send, client)
         client._process_response(result, self.session)
         return result, to_send
 
     def execute_unack(
-        self, cmd: MutableMapping[str, Any], docs: List[Mapping[str, Any]], client: MongoClient
-    ) -> List[Mapping[str, Any]]:
+        self, cmd: MutableMapping[str, Any], docs: list[Mapping[str, Any]], client: MongoClient
+    ) -> list[Mapping[str, Any]]:
         request_id, msg, to_send = self.__batch_command(cmd, docs)
         # Though this isn't strictly a "legacy" write, the helper
         # handles publishing commands and sending our message
         # without receiving a result. Send 0 for max_doc_size
         # to disable size checking. Size checking is handled while
         # the documents are encoded to BSON.
-        self.unack_write(cmd, request_id, msg, 0, to_send)
+        self.unack_write(cmd, request_id, msg, 0, to_send, client)
         return to_send
 
     @property
@@ -1009,34 +1002,83 @@ class _BulkWriteContext:
         request_id: int,
         msg: bytes,
         max_doc_size: int,
-        docs: List[Mapping[str, Any]],
+        docs: list[Mapping[str, Any]],
+        client: MongoClient,
     ) -> Optional[Mapping[str, Any]]:
         """A proxy for Connection.unack_write that handles event publishing."""
+        if _COMMAND_LOGGER.isEnabledFor(logging.DEBUG):
+            _debug_log(
+                _COMMAND_LOGGER,
+                clientId=client._topology_settings._topology_id,
+                message=_CommandStatusMessage.STARTED,
+                command=cmd,
+                commandName=next(iter(cmd)),
+                databaseName=self.db_name,
+                requestId=request_id,
+                operationId=request_id,
+                driverConnectionId=self.conn.id,
+                serverConnectionId=self.conn.server_connection_id,
+                serverHost=self.conn.address[0],
+                serverPort=self.conn.address[1],
+                serviceId=self.conn.service_id,
+            )
         if self.publish:
-            assert self.start_time is not None
-            duration = datetime.datetime.now() - self.start_time
             cmd = self._start(cmd, request_id, docs)
-            start = datetime.datetime.now()
         try:
             result = self.conn.unack_write(msg, max_doc_size)  # type: ignore[func-returns-value]
+            duration = datetime.datetime.now() - self.start_time
+            if result is not None:
+                reply = _convert_write_result(self.name, cmd, result)
+            else:
+                # Comply with APM spec.
+                reply = {"ok": 1}
+                if _COMMAND_LOGGER.isEnabledFor(logging.DEBUG):
+                    _debug_log(
+                        _COMMAND_LOGGER,
+                        clientId=client._topology_settings._topology_id,
+                        message=_CommandStatusMessage.SUCCEEDED,
+                        durationMS=duration,
+                        reply=reply,
+                        commandName=next(iter(cmd)),
+                        databaseName=self.db_name,
+                        requestId=request_id,
+                        operationId=request_id,
+                        driverConnectionId=self.conn.id,
+                        serverConnectionId=self.conn.server_connection_id,
+                        serverHost=self.conn.address[0],
+                        serverPort=self.conn.address[1],
+                        serviceId=self.conn.service_id,
+                    )
             if self.publish:
-                duration = (datetime.datetime.now() - start) + duration
-                if result is not None:
-                    reply = _convert_write_result(self.name, cmd, result)
-                else:
-                    # Comply with APM spec.
-                    reply = {"ok": 1}
                 self._succeed(request_id, reply, duration)
         except Exception as exc:
+            duration = datetime.datetime.now() - self.start_time
+            if isinstance(exc, OperationFailure):
+                failure: _DocumentOut = _convert_write_result(self.name, cmd, exc.details)  # type: ignore[arg-type]
+            elif isinstance(exc, NotPrimaryError):
+                failure = exc.details  # type: ignore[assignment]
+            else:
+                failure = _convert_exception(exc)
+            if _COMMAND_LOGGER.isEnabledFor(logging.DEBUG):
+                _debug_log(
+                    _COMMAND_LOGGER,
+                    clientId=client._topology_settings._topology_id,
+                    message=_CommandStatusMessage.FAILED,
+                    durationMS=duration,
+                    failure=failure,
+                    commandName=next(iter(cmd)),
+                    databaseName=self.db_name,
+                    requestId=request_id,
+                    operationId=request_id,
+                    driverConnectionId=self.conn.id,
+                    serverConnectionId=self.conn.server_connection_id,
+                    serverHost=self.conn.address[0],
+                    serverPort=self.conn.address[1],
+                    serviceId=self.conn.service_id,
+                    isServerSideError=isinstance(exc, OperationFailure),
+                )
             if self.publish:
                 assert self.start_time is not None
-                duration = (datetime.datetime.now() - start) + duration
-                if isinstance(exc, OperationFailure):
-                    failure: _DocumentOut = _convert_write_result(self.name, cmd, exc.details)  # type: ignore[arg-type]
-                elif isinstance(exc, NotPrimaryError):
-                    failure = exc.details  # type: ignore[assignment]
-                else:
-                    failure = _convert_exception(exc)
                 self._fail(request_id, failure, duration)
             raise
         finally:
@@ -1049,26 +1091,77 @@ class _BulkWriteContext:
         cmd: MutableMapping[str, Any],
         request_id: int,
         msg: bytes,
-        docs: List[Mapping[str, Any]],
-    ) -> Dict[str, Any]:
+        docs: list[Mapping[str, Any]],
+        client: MongoClient,
+    ) -> dict[str, Any]:
         """A proxy for SocketInfo.write_command that handles event publishing."""
+        cmd[self.field] = docs
+        if _COMMAND_LOGGER.isEnabledFor(logging.DEBUG):
+            _debug_log(
+                _COMMAND_LOGGER,
+                clientId=client._topology_settings._topology_id,
+                message=_CommandStatusMessage.STARTED,
+                command=cmd,
+                commandName=next(iter(cmd)),
+                databaseName=self.db_name,
+                requestId=request_id,
+                operationId=request_id,
+                driverConnectionId=self.conn.id,
+                serverConnectionId=self.conn.server_connection_id,
+                serverHost=self.conn.address[0],
+                serverPort=self.conn.address[1],
+                serviceId=self.conn.service_id,
+            )
         if self.publish:
-            assert self.start_time is not None
-            duration = datetime.datetime.now() - self.start_time
             self._start(cmd, request_id, docs)
-            start = datetime.datetime.now()
         try:
             reply = self.conn.write_command(request_id, msg, self.codec)
+            duration = datetime.datetime.now() - self.start_time
+            if _COMMAND_LOGGER.isEnabledFor(logging.DEBUG):
+                _debug_log(
+                    _COMMAND_LOGGER,
+                    clientId=client._topology_settings._topology_id,
+                    message=_CommandStatusMessage.SUCCEEDED,
+                    durationMS=duration,
+                    reply=reply,
+                    commandName=next(iter(cmd)),
+                    databaseName=self.db_name,
+                    requestId=request_id,
+                    operationId=request_id,
+                    driverConnectionId=self.conn.id,
+                    serverConnectionId=self.conn.server_connection_id,
+                    serverHost=self.conn.address[0],
+                    serverPort=self.conn.address[1],
+                    serviceId=self.conn.service_id,
+                )
             if self.publish:
-                duration = (datetime.datetime.now() - start) + duration
                 self._succeed(request_id, reply, duration)
         except Exception as exc:
+            duration = datetime.datetime.now() - self.start_time
+            if isinstance(exc, (NotPrimaryError, OperationFailure)):
+                failure: _DocumentOut = exc.details  # type: ignore[assignment]
+            else:
+                failure = _convert_exception(exc)
+            if _COMMAND_LOGGER.isEnabledFor(logging.DEBUG):
+                _debug_log(
+                    _COMMAND_LOGGER,
+                    clientId=client._topology_settings._topology_id,
+                    message=_CommandStatusMessage.FAILED,
+                    durationMS=duration,
+                    failure=failure,
+                    commandName=next(iter(cmd)),
+                    databaseName=self.db_name,
+                    requestId=request_id,
+                    operationId=request_id,
+                    driverConnectionId=self.conn.id,
+                    serverConnectionId=self.conn.server_connection_id,
+                    serverHost=self.conn.address[0],
+                    serverPort=self.conn.address[1],
+                    serviceId=self.conn.service_id,
+                    isServerSideError=isinstance(exc, OperationFailure),
+                )
+
             if self.publish:
-                duration = (datetime.datetime.now() - start) + duration
-                if isinstance(exc, (NotPrimaryError, OperationFailure)):
-                    failure: _DocumentOut = exc.details  # type: ignore[assignment]
-                else:
-                    failure = _convert_exception(exc)
                 self._fail(request_id, failure, duration)
             raise
         finally:
@@ -1076,7 +1169,7 @@ class _BulkWriteContext:
         return reply
 
     def _start(
-        self, cmd: MutableMapping[str, Any], request_id: int, docs: List[Mapping[str, Any]]
+        self, cmd: MutableMapping[str, Any], request_id: int, docs: list[Mapping[str, Any]]
     ) -> MutableMapping[str, Any]:
         """Publish a CommandStartedEvent."""
         cmd[self.field] = docs
@@ -1085,6 +1178,7 @@ class _BulkWriteContext:
             self.db_name,
             request_id,
             self.conn.address,
+            self.conn.server_connection_id,
             self.op_id,
             self.conn.service_id,
         )
@@ -1098,8 +1192,10 @@ class _BulkWriteContext:
             self.name,
             request_id,
             self.conn.address,
+            self.conn.server_connection_id,
             self.op_id,
             self.conn.service_id,
+            database_name=self.db_name,
         )
 
     def _fail(self, request_id: int, failure: _DocumentOut, duration: timedelta) -> None:
@@ -1110,8 +1206,10 @@ class _BulkWriteContext:
             self.name,
             request_id,
             self.conn.address,
+            self.conn.server_connection_id,
             self.op_id,
             self.conn.service_id,
+            database_name=self.db_name,
         )
 
 
@@ -1126,8 +1224,8 @@ class _EncryptedBulkWriteContext(_BulkWriteContext):
     __slots__ = ()
 
     def __batch_command(
-        self, cmd: MutableMapping[str, Any], docs: List[Mapping[str, Any]]
-    ) -> Tuple[Dict[str, Any], List[Mapping[str, Any]]]:
+        self, cmd: MutableMapping[str, Any], docs: list[Mapping[str, Any]]
+    ) -> tuple[dict[str, Any], list[Mapping[str, Any]]]:
         namespace = self.db_name + ".$cmd"
         msg, to_send = _encode_batched_write_command(
             namespace, self.op_type, cmd, docs, self.codec, self
@@ -1137,12 +1235,12 @@ class _EncryptedBulkWriteContext(_BulkWriteContext):
 
         # Chop off the OP_QUERY header to get a properly batched write command.
         cmd_start = msg.index(b"\x00", 4) + 9
-        cmd = _inflate_bson(memoryview(msg)[cmd_start:], DEFAULT_RAW_BSON_OPTIONS)
-        return cmd, to_send
+        outgoing = _inflate_bson(memoryview(msg)[cmd_start:], DEFAULT_RAW_BSON_OPTIONS)
+        return outgoing, to_send
 
     def execute(
-        self, cmd: MutableMapping[str, Any], docs: List[Mapping[str, Any]], client: MongoClient
-    ) -> Tuple[Mapping[str, Any], List[Mapping[str, Any]]]:
+        self, cmd: MutableMapping[str, Any], docs: list[Mapping[str, Any]], client: MongoClient
+    ) -> tuple[Mapping[str, Any], list[Mapping[str, Any]]]:
         batched_cmd, to_send = self.__batch_command(cmd, docs)
         result: Mapping[str, Any] = self.conn.command(
             self.db_name, batched_cmd, codec_options=self.codec, session=self.session, client=client
@@ -1150,8 +1248,8 @@ class _EncryptedBulkWriteContext(_BulkWriteContext):
         return result, to_send
 
     def execute_unack(
-        self, cmd: MutableMapping[str, Any], docs: List[Mapping[str, Any]], client: MongoClient
-    ) -> List[Mapping[str, Any]]:
+        self, cmd: MutableMapping[str, Any], docs: list[Mapping[str, Any]], client: MongoClient
+    ) -> list[Mapping[str, Any]]:
         batched_cmd, to_send = self.__batch_command(cmd, docs)
         self.conn.command(
             self.db_name,
@@ -1196,12 +1294,12 @@ _OP_MSG_MAP = {
 def _batched_op_msg_impl(
     operation: int,
     command: Mapping[str, Any],
-    docs: List[Mapping[str, Any]],
+    docs: list[Mapping[str, Any]],
     ack: bool,
     opts: CodecOptions,
     ctx: _BulkWriteContext,
     buf: _BytesIO,
-) -> Tuple[List[Mapping[str, Any]], int]:
+) -> tuple[list[Mapping[str, Any]], int]:
     """Create a batched OP_MSG write."""
     max_bson_size = ctx.max_bson_size
     max_write_batch_size = ctx.max_write_batch_size
@@ -1223,7 +1321,7 @@ def _batched_op_msg_impl(
     try:
         buf.write(_OP_MSG_MAP[operation])
     except KeyError:
-        raise InvalidOperation("Unknown command")
+        raise InvalidOperation("Unknown command") from None
 
     to_send = []
     idx = 0
@@ -1264,11 +1362,11 @@ def _batched_op_msg_impl(
 def _encode_batched_op_msg(
     operation: int,
     command: Mapping[str, Any],
-    docs: List[Mapping[str, Any]],
+    docs: list[Mapping[str, Any]],
     ack: bool,
     opts: CodecOptions,
     ctx: _BulkWriteContext,
-) -> Tuple[bytes, List[Mapping[str, Any]]]:
+) -> tuple[bytes, list[Mapping[str, Any]]]:
     """Encode the next batched insert, update, or delete operation
     as OP_MSG.
     """
@@ -1279,17 +1377,17 @@ def _encode_batched_op_msg(
 
 
 if _use_c:
-    _encode_batched_op_msg = _cmessage._encode_batched_op_msg  # noqa: F811
+    _encode_batched_op_msg = _cmessage._encode_batched_op_msg
 
 
 def _batched_op_msg_compressed(
     operation: int,
     command: Mapping[str, Any],
-    docs: List[Mapping[str, Any]],
+    docs: list[Mapping[str, Any]],
     ack: bool,
     opts: CodecOptions,
     ctx: _BulkWriteContext,
-) -> Tuple[int, bytes, List[Mapping[str, Any]]]:
+) -> tuple[int, bytes, list[Mapping[str, Any]]]:
     """Create the next batched insert, update, or delete operation
     with OP_MSG, compressed.
     """
@@ -1303,11 +1401,11 @@ def _batched_op_msg_compressed(
 def _batched_op_msg(
     operation: int,
     command: Mapping[str, Any],
-    docs: List[Mapping[str, Any]],
+    docs: list[Mapping[str, Any]],
     ack: bool,
     opts: CodecOptions,
     ctx: _BulkWriteContext,
-) -> Tuple[int, bytes, List[Mapping[str, Any]]]:
+) -> tuple[int, bytes, list[Mapping[str, Any]]]:
     """OP_MSG implementation entry point."""
     buf = _BytesIO()
 
@@ -1329,17 +1427,17 @@ def _batched_op_msg(
 
 
 if _use_c:
-    _batched_op_msg = _cmessage._batched_op_msg  # noqa: F811
+    _batched_op_msg = _cmessage._batched_op_msg
 
 
 def _do_batched_op_msg(
     namespace: str,
     operation: int,
     command: MutableMapping[str, Any],
-    docs: List[Mapping[str, Any]],
+    docs: list[Mapping[str, Any]],
     opts: CodecOptions,
     ctx: _BulkWriteContext,
-) -> Tuple[int, bytes, List[Mapping[str, Any]]]:
+) -> tuple[int, bytes, list[Mapping[str, Any]]]:
     """Create the next batched insert, update, or delete operation
     using OP_MSG.
     """
@@ -1360,10 +1458,10 @@ def _encode_batched_write_command(
     namespace: str,
     operation: int,
     command: MutableMapping[str, Any],
-    docs: List[Mapping[str, Any]],
+    docs: list[Mapping[str, Any]],
     opts: CodecOptions,
     ctx: _BulkWriteContext,
-) -> Tuple[bytes, List[Mapping[str, Any]]]:
+) -> tuple[bytes, list[Mapping[str, Any]]]:
     """Encode the next batched insert, update, or delete command."""
     buf = _BytesIO()
 
@@ -1372,18 +1470,18 @@ def _encode_batched_write_command(
 
 
 if _use_c:
-    _encode_batched_write_command = _cmessage._encode_batched_write_command  # noqa: F811
+    _encode_batched_write_command = _cmessage._encode_batched_write_command
 
 
 def _batched_write_command_impl(
     namespace: str,
     operation: int,
     command: MutableMapping[str, Any],
-    docs: List[Mapping[str, Any]],
+    docs: list[Mapping[str, Any]],
     opts: CodecOptions,
     ctx: _BulkWriteContext,
     buf: _BytesIO,
-) -> Tuple[List[Mapping[str, Any]], int]:
+) -> tuple[list[Mapping[str, Any]], int]:
     """Create a batched OP_QUERY write command."""
     max_bson_size = ctx.max_bson_size
     max_write_batch_size = ctx.max_write_batch_size
@@ -1411,7 +1509,7 @@ def _batched_write_command_impl(
     try:
         buf.write(_OP_MAP[operation])
     except KeyError:
-        raise InvalidOperation("Unknown command")
+        raise InvalidOperation("Unknown command") from None
 
     # Where to write list document length
     list_start = buf.tell() - 4
@@ -1468,7 +1566,7 @@ class _OpReply:
 
     def raw_response(
         self, cursor_id: Optional[int] = None, user_fields: Optional[Mapping[str, Any]] = None
-    ) -> List[bytes]:
+    ) -> list[bytes]:
         """Check the response header from the database, without decoding BSON.
 
         Check the response for errors and unpack.
@@ -1476,8 +1574,7 @@ class _OpReply:
         Can raise CursorNotFound, NotPrimaryError, ExecutionTimeout, or
         OperationFailure.
 
-        :Parameters:
-          - `cursor_id` (optional): cursor_id we sent to get this response -
+        :param cursor_id: cursor_id we sent to get this response -
             used for raising an informative exception when we get cursor id not
             valid at server response.
         """
@@ -1517,7 +1614,7 @@ class _OpReply:
         codec_options: CodecOptions = _UNICODE_REPLACE_CODEC_OPTIONS,
         user_fields: Optional[Mapping[str, Any]] = None,
         legacy_response: bool = False,
-    ) -> List[Dict[str, Any]]:
+    ) -> list[dict[str, Any]]:
         """Unpack a response from the database and decode the BSON document(s).
 
         Check the response for errors and unpack, returning a dictionary
@@ -1526,13 +1623,12 @@ class _OpReply:
         Can raise CursorNotFound, NotPrimaryError, ExecutionTimeout, or
         OperationFailure.
 
-        :Parameters:
-          - `cursor_id` (optional): cursor_id we sent to get this response -
+        :param cursor_id: cursor_id we sent to get this response -
             used for raising an informative exception when we get cursor id not
             valid at server response
-          - `codec_options` (optional): an instance of
+        :param codec_options: an instance of
             :class:`~bson.codec_options.CodecOptions`
-          - `user_fields` (optional): Response fields that should be decoded
+        :param user_fields: Response fields that should be decoded
             using the TypeDecoders from codec_options, passed to
             bson._decode_all_selective.
         """
@@ -1541,7 +1637,7 @@ class _OpReply:
             return bson.decode_all(self.documents, codec_options)
         return bson._decode_all_selective(self.documents, codec_options, user_fields)
 
-    def command_response(self, codec_options: CodecOptions) -> Dict[str, Any]:
+    def command_response(self, codec_options: CodecOptions) -> dict[str, Any]:
         """Unpack a command response."""
         docs = self.unpack_response(codec_options=codec_options)
         assert self.number_returned == 1
@@ -1587,8 +1683,8 @@ class _OpMsg:
     def raw_response(
         self,
         cursor_id: Optional[int] = None,
-        user_fields: Optional[Mapping[str, Any]] = {},  # noqa: B006
-    ) -> List[Mapping[str, Any]]:
+        user_fields: Optional[Mapping[str, Any]] = {},
+    ) -> list[Mapping[str, Any]]:
         """
         cursor_id is ignored
         user_fields is used to determine which fields must not be decoded
@@ -1604,14 +1700,13 @@ class _OpMsg:
         codec_options: CodecOptions = _UNICODE_REPLACE_CODEC_OPTIONS,
         user_fields: Optional[Mapping[str, Any]] = None,
         legacy_response: bool = False,
-    ) -> List[Dict[str, Any]]:
+    ) -> list[dict[str, Any]]:
         """Unpack a OP_MSG command response.
 
-        :Parameters:
-          - `cursor_id` (optional): Ignored, for compatibility with _OpReply.
-          - `codec_options` (optional): an instance of
+        :param cursor_id: Ignored, for compatibility with _OpReply.
+        :param codec_options: an instance of
             :class:`~bson.codec_options.CodecOptions`
-          - `user_fields` (optional): Response fields that should be decoded
+        :param user_fields: Response fields that should be decoded
             using the TypeDecoders from codec_options, passed to
             bson._decode_all_selective.
         """
@@ -1619,7 +1714,7 @@ class _OpMsg:
         assert not legacy_response
         return bson._decode_all_selective(self.payload_document, codec_options, user_fields)
 
-    def command_response(self, codec_options: CodecOptions) -> Dict[str, Any]:
+    def command_response(self, codec_options: CodecOptions) -> dict[str, Any]:
         """Unpack a command response."""
         return self.unpack_response(codec_options=codec_options)[0]
 
@@ -1652,7 +1747,7 @@ class _OpMsg:
         return cls(flags, payload_document)
 
 
-_UNPACK_REPLY: Dict[int, Callable[[bytes], Union[_OpReply, _OpMsg]]] = {
+_UNPACK_REPLY: dict[int, Callable[[bytes], Union[_OpReply, _OpMsg]]] = {
     _OpReply.OP_CODE: _OpReply.unpack,
     _OpMsg.OP_CODE: _OpMsg.unpack,
 }
