@@ -17,18 +17,23 @@ from __future__ import annotations
 
 import ipaddress
 import random
-from typing import Any, List, Optional, Tuple, Union
-
-try:
-    from dns import resolver
-
-    _HAVE_DNSPYTHON = True
-except ImportError:
-    _HAVE_DNSPYTHON = False
+from typing import TYPE_CHECKING, Any, Optional, Union
 
 from pymongo.common import CONNECT_TIMEOUT
 from pymongo.errors import ConfigurationError
 
+if TYPE_CHECKING:
+    from dns import resolver
+
+
+def _have_dnspython() -> bool:
+    try:
+        import dns  # noqa: F401
+
+        return True
+    except ImportError:
+        return False
+
 
 # dnspython can return bytes or str from various parts
 # of its API depending on version. We always want str.
@@ -40,6 +45,8 @@ def maybe_decode(text: Union[str, bytes]) -> str:
 
 # PYTHON-2667 Lazily call dns.resolver methods for compatibility with eventlet.
 def _resolve(*args: Any, **kwargs: Any) -> resolver.Answer:
+    from dns import resolver
+
     if hasattr(resolver, "resolve"):
         # dnspython >= 2
         return resolver.resolve(*args, **kwargs)
@@ -75,19 +82,21 @@ class _SrvResolver:
         try:
             self.__plist = self.__fqdn.split(".")[1:]
         except Exception:
-            raise ConfigurationError(_INVALID_HOST_MSG % (fqdn,))
+            raise ConfigurationError(_INVALID_HOST_MSG % (fqdn,)) from None
         self.__slen = len(self.__plist)
         if self.__slen < 2:
             raise ConfigurationError(_INVALID_HOST_MSG % (fqdn,))
 
     def get_options(self) -> Optional[str]:
+        from dns import resolver
+
         try:
             results = _resolve(self.__fqdn, "TXT", lifetime=self.__connect_timeout)
         except (resolver.NoAnswer, resolver.NXDOMAIN):
             # No TXT records
             return None
         except Exception as exc:
-            raise ConfigurationError(str(exc))
+            raise ConfigurationError(str(exc)) from None
         if len(results) > 1:
             raise ConfigurationError("Only one TXT record is supported")
         return (b"&".join([b"".join(res.strings) for res in results])).decode("utf-8")
@@ -102,12 +111,12 @@ class _SrvResolver:
                 # Raise the original error.
                 raise
             # Else, raise all errors as ConfigurationError.
-            raise ConfigurationError(str(exc))
+            raise ConfigurationError(str(exc)) from None
         return results
 
     def _get_srv_response_and_hosts(
         self, encapsulate_errors: bool
-    ) -> Tuple[resolver.Answer, List[Tuple[str, Any]]]:
+    ) -> tuple[resolver.Answer, list[tuple[str, Any]]]:
         results = self._resolve_uri(encapsulate_errors)
 
         # Construct address tuples
@@ -120,18 +129,18 @@ class _SrvResolver:
             try:
                 nlist = node[0].lower().split(".")[1:][-self.__slen :]
             except Exception:
-                raise ConfigurationError(f"Invalid SRV host: {node[0]}")
+                raise ConfigurationError(f"Invalid SRV host: {node[0]}") from None
             if self.__plist != nlist:
                 raise ConfigurationError(f"Invalid SRV host: {node[0]}")
         if self.__srv_max_hosts:
             nodes = random.sample(nodes, min(self.__srv_max_hosts, len(nodes)))
         return results, nodes
 
-    def get_hosts(self) -> List[Tuple[str, Any]]:
+    def get_hosts(self) -> list[tuple[str, Any]]:
         _, nodes = self._get_srv_response_and_hosts(True)
         return nodes
 
-    def get_hosts_and_min_ttl(self) -> Tuple[List[Tuple[str, Any]], int]:
+    def get_hosts_and_min_ttl(self) -> tuple[list[tuple[str, Any]], int]:
         results, nodes = self._get_srv_response_and_hosts(False)
         rrset = results.rrset
         ttl = rrset.ttl if rrset else 0
