@@ -13,18 +13,17 @@
 # permissions and limitations under the License.
 
 """Represent a deployment of MongoDB servers."""
+from __future__ import annotations
 
 from random import sample
 from typing import (
     Any,
     Callable,
-    Dict,
     List,
     Mapping,
     MutableMapping,
     NamedTuple,
     Optional,
-    Tuple,
     cast,
 )
 
@@ -52,7 +51,7 @@ class _TopologyType(NamedTuple):
 TOPOLOGY_TYPE = _TopologyType(*range(6))
 
 # Topologies compatible with SRV record polling.
-SRV_POLLING_TOPOLOGIES: Tuple[int, int] = (TOPOLOGY_TYPE.Unknown, TOPOLOGY_TYPE.Sharded)
+SRV_POLLING_TOPOLOGIES: tuple[int, int] = (TOPOLOGY_TYPE.Unknown, TOPOLOGY_TYPE.Sharded)
 
 
 _ServerSelector = Callable[[List[ServerDescription]], List[ServerDescription]]
@@ -62,7 +61,7 @@ class TopologyDescription:
     def __init__(
         self,
         topology_type: int,
-        server_descriptions: Dict[_Address, ServerDescription],
+        server_descriptions: dict[_Address, ServerDescription],
         replica_set_name: Optional[str],
         max_set_version: Optional[int],
         max_election_id: Optional[ObjectId],
@@ -70,14 +69,13 @@ class TopologyDescription:
     ) -> None:
         """Representation of a deployment of MongoDB servers.
 
-        :Parameters:
-          - `topology_type`: initial type
-          - `server_descriptions`: dict of (address, ServerDescription) for
+        :param topology_type: initial type
+        :param server_descriptions: dict of (address, ServerDescription) for
             all seeds
-          - `replica_set_name`: replica set name or None
-          - `max_set_version`: greatest setVersion seen from a primary, or None
-          - `max_election_id`: greatest electionId seen from a primary, or None
-          - `topology_settings`: a TopologySettings
+        :param replica_set_name: replica set name or None
+        :param max_set_version: greatest setVersion seen from a primary, or None
+        :param max_election_id: greatest electionId seen from a primary, or None
+        :param topology_settings: a TopologySettings
         """
         self._topology_type = topology_type
         self._replica_set_name = replica_set_name
@@ -169,12 +167,12 @@ class TopologyDescription:
     def has_server(self, address: _Address) -> bool:
         return address in self._server_descriptions
 
-    def reset_server(self, address: _Address) -> "TopologyDescription":
+    def reset_server(self, address: _Address) -> TopologyDescription:
         """A copy of this description, with one server marked Unknown."""
         unknown_sd = self._server_descriptions[address].to_unknown()
         return updated_topology_description(self, unknown_sd)
 
-    def reset(self) -> "TopologyDescription":
+    def reset(self) -> TopologyDescription:
         """A copy of this description, with all servers marked Unknown."""
         if self._topology_type == TOPOLOGY_TYPE.ReplicaSetWithPrimary:
             topology_type = TOPOLOGY_TYPE.ReplicaSetNoPrimary
@@ -193,8 +191,8 @@ class TopologyDescription:
             self._topology_settings,
         )
 
-    def server_descriptions(self) -> Dict[_Address, ServerDescription]:
-        """Dict of (address,
+    def server_descriptions(self) -> dict[_Address, ServerDescription]:
+        """dict of (address,
         :class:`~pymongo.server_description.ServerDescription`).
         """
         return self._server_descriptions.copy()
@@ -233,7 +231,7 @@ class TopologyDescription:
         return self._ls_timeout_minutes
 
     @property
-    def known_servers(self) -> List[ServerDescription]:
+    def known_servers(self) -> list[ServerDescription]:
         """List of Servers of types besides Unknown."""
         return [s for s in self._server_descriptions.values() if s.is_server_type_known]
 
@@ -243,7 +241,7 @@ class TopologyDescription:
         return any(s for s in self._server_descriptions.values() if s.is_server_type_known)
 
     @property
-    def readable_servers(self) -> List[ServerDescription]:
+    def readable_servers(self) -> list[ServerDescription]:
         """List of readable Servers."""
         return [s for s in self._server_descriptions.values() if s.is_readable]
 
@@ -264,11 +262,17 @@ class TopologyDescription:
     def srv_max_hosts(self) -> int:
         return self._topology_settings._srv_max_hosts
 
-    def _apply_local_threshold(self, selection: Optional[Selection]) -> List[ServerDescription]:
+    def _apply_local_threshold(self, selection: Optional[Selection]) -> list[ServerDescription]:
         if not selection:
             return []
+        round_trip_times: list[float] = []
+        for server in selection.server_descriptions:
+            if server.round_trip_time is None:
+                config_err_msg = f"round_trip_time for server {server.address} is unexpectedly None: {self}, servers: {selection.server_descriptions}"
+                raise ConfigurationError(config_err_msg)
+            round_trip_times.append(server.round_trip_time)
         # Round trip time in seconds.
-        fastest = min(cast(float, s.round_trip_time) for s in selection.server_descriptions)
+        fastest = min(round_trip_times)
         threshold = self._topology_settings.local_threshold_ms / 1000.0
         return [
             s
@@ -281,15 +285,14 @@ class TopologyDescription:
         selector: Any,
         address: Optional[_Address] = None,
         custom_selector: Optional[_ServerSelector] = None,
-    ) -> List[ServerDescription]:
+    ) -> list[ServerDescription]:
         """List of servers matching the provided selector(s).
 
-        :Parameters:
-          - `selector`: a callable that takes a Selection as input and returns
+        :param selector: a callable that takes a Selection as input and returns
             a Selection as output. For example, an instance of a read
             preference from :mod:`~pymongo.read_preferences`.
-          - `address` (optional): A server address to select.
-          - `custom_selector` (optional): A callable that augments server
+        :param address: A server address to select.
+        :param custom_selector: A callable that augments server
             selection rules. Accepts a list of
             :class:`~pymongo.server_description.ServerDescription` objects and
             return a list of server descriptions that should be considered
@@ -334,8 +337,7 @@ class TopologyDescription:
         """Does this topology have any readable servers available matching the
         given read preference?
 
-        :Parameters:
-          - `read_preference`: an instance of a read preference from
+        :param read_preference: an instance of a read preference from
             :mod:`~pymongo.read_preferences`. Defaults to
             :attr:`~pymongo.read_preferences.ReadPreference.PRIMARY`.
 
@@ -382,12 +384,11 @@ _SERVER_TYPE_TO_TOPOLOGY_TYPE = {
 
 def updated_topology_description(
     topology_description: TopologyDescription, server_description: ServerDescription
-) -> "TopologyDescription":
+) -> TopologyDescription:
     """Return an updated copy of a TopologyDescription.
 
-    :Parameters:
-      - `topology_description`: the current TopologyDescription
-      - `server_description`: a new ServerDescription that resulted from
+    :param topology_description: the current TopologyDescription
+    :param server_description: a new ServerDescription that resulted from
         a hello call
 
     Called after attempting (successfully or not) to call hello on the
@@ -486,13 +487,12 @@ def updated_topology_description(
 
 
 def _updated_topology_description_srv_polling(
-    topology_description: TopologyDescription, seedlist: List[Tuple[str, Any]]
+    topology_description: TopologyDescription, seedlist: list[tuple[str, Any]]
 ) -> TopologyDescription:
     """Return an updated copy of a TopologyDescription.
 
-    :Parameters:
-      - `topology_description`: the current TopologyDescription
-      - `seedlist`: a list of new seeds new ServerDescription that resulted from
+    :param topology_description: the current TopologyDescription
+    :param seedlist: a list of new seeds new ServerDescription that resulted from
         a hello call
     """
     assert topology_description.topology_type in SRV_POLLING_TOPOLOGIES
@@ -535,7 +535,7 @@ def _update_rs_from_primary(
     server_description: ServerDescription,
     max_set_version: Optional[int],
     max_election_id: Optional[ObjectId],
-) -> Tuple[int, Optional[str], Optional[int], Optional[ObjectId]]:
+) -> tuple[int, Optional[str], Optional[int], Optional[ObjectId]]:
     """Update topology description from a primary's hello response.
 
     Pass in a dict of ServerDescriptions, current replica set name, the
@@ -555,8 +555,8 @@ def _update_rs_from_primary(
         return _check_has_primary(sds), replica_set_name, max_set_version, max_election_id
 
     if server_description.max_wire_version is None or server_description.max_wire_version < 17:
-        new_election_tuple: Tuple = (server_description.set_version, server_description.election_id)
-        max_election_tuple: Tuple = (max_set_version, max_election_id)
+        new_election_tuple: tuple = (server_description.set_version, server_description.election_id)
+        max_election_tuple: tuple = (max_set_version, max_election_id)
         if None not in new_election_tuple:
             if None not in max_election_tuple and new_election_tuple < max_election_tuple:
                 # Stale primary, set to type Unknown.
@@ -587,7 +587,6 @@ def _update_rs_from_primary(
             server.server_type is SERVER_TYPE.RSPrimary
             and server.address != server_description.address
         ):
-
             # Reset old primary's type to Unknown.
             sds[server.address] = server.to_unknown()
 
@@ -635,7 +634,7 @@ def _update_rs_no_primary_from_member(
     sds: MutableMapping[_Address, ServerDescription],
     replica_set_name: Optional[str],
     server_description: ServerDescription,
-) -> Tuple[int, Optional[str]]:
+) -> tuple[int, Optional[str]]:
     """RS without known primary. Update from a non-primary's response.
 
     Pass in a dict of ServerDescriptions, current replica set name, and the
@@ -673,5 +672,5 @@ def _check_has_primary(sds: Mapping[_Address, ServerDescription]) -> int:
     for s in sds.values():
         if s.server_type == SERVER_TYPE.RSPrimary:
             return TOPOLOGY_TYPE.ReplicaSetWithPrimary
-    else:
+    else:  # noqa: PLW0120
         return TOPOLOGY_TYPE.ReplicaSetNoPrimary
