@@ -24,20 +24,16 @@ from itertools import islice
 from typing import (
     TYPE_CHECKING,
     Any,
-    Dict,
     Iterator,
-    List,
     Mapping,
     NoReturn,
     Optional,
-    Tuple,
     Type,
     Union,
 )
 
 from bson.objectid import ObjectId
 from bson.raw_bson import RawBSONDocument
-from bson.son import SON
 from pymongo import _csot, common
 from pymongo.client_session import ClientSession, _validate_session_write_concern
 from pymongo.common import (
@@ -76,7 +72,7 @@ _BAD_VALUE: int = 2
 _UNKNOWN_ERROR: int = 8
 _WRITE_CONCERN_ERROR: int = 64
 
-_COMMANDS: Tuple[str, str, str] = ("insert", "update", "delete")
+_COMMANDS: tuple[str, str, str] = ("insert", "update", "delete")
 
 
 class _Run:
@@ -85,25 +81,23 @@ class _Run:
     def __init__(self, op_type: int) -> None:
         """Initialize a new Run object."""
         self.op_type: int = op_type
-        self.index_map: List[int] = []
-        self.ops: List[Any] = []
+        self.index_map: list[int] = []
+        self.ops: list[Any] = []
         self.idx_offset: int = 0
 
     def index(self, idx: int) -> int:
         """Get the original index of an operation in this run.
 
-        :Parameters:
-          - `idx`: The Run index that maps to the original index.
+        :param idx: The Run index that maps to the original index.
         """
         return self.index_map[idx]
 
     def add(self, original_index: int, operation: Any) -> None:
         """Add an operation to this Run instance.
 
-        :Parameters:
-          - `original_index`: The original index of this operation
+        :param original_index: The original index of this operation
             within a larger bulk operation.
-          - `operation`: The operation document.
+        :param operation: The operation document.
         """
         self.index_map.append(original_index)
         self.ops.append(operation)
@@ -155,8 +149,19 @@ def _merge_command(
 
 def _raise_bulk_write_error(full_result: _DocumentOut) -> NoReturn:
     """Raise a BulkWriteError from the full bulk api result."""
+    # retryWrites on MMAPv1 should raise an actionable error.
     if full_result["writeErrors"]:
         full_result["writeErrors"].sort(key=lambda error: error["index"])
+        err = full_result["writeErrors"][0]
+        code = err["code"]
+        msg = err["errmsg"]
+        if code == 20 and msg.startswith("Transaction numbers"):
+            errmsg = (
+                "This MongoDB deployment does not support "
+                "retryable writes. Please add retryWrites=false "
+                "to your connection string."
+            )
+            raise OperationFailure(errmsg, code, full_result)
     raise BulkWriteError(full_result)
 
 
@@ -182,7 +187,7 @@ class _Bulk:
             common.validate_is_document_type("let", self.let)
         self.comment: Optional[str] = comment
         self.ordered = ordered
-        self.ops: List[Tuple[int, Mapping[str, Any]]] = []
+        self.ops: list[tuple[int, Mapping[str, Any]]] = []
         self.executed = False
         self.bypass_doc_val = bypass_document_validation
         self.uses_collation = False
@@ -219,12 +224,12 @@ class _Bulk:
         multi: bool = False,
         upsert: bool = False,
         collation: Optional[Mapping[str, Any]] = None,
-        array_filters: Optional[List[Mapping[str, Any]]] = None,
-        hint: Union[str, SON[str, Any], None] = None,
+        array_filters: Optional[list[Mapping[str, Any]]] = None,
+        hint: Union[str, dict[str, Any], None] = None,
     ) -> None:
         """Create an update document and add it to the list of ops."""
         validate_ok_for_update(update)
-        cmd: Dict[str, Any] = dict(
+        cmd: dict[str, Any] = dict(  # noqa: C406
             [("q", selector), ("u", update), ("multi", multi), ("upsert", upsert)]
         )
         if collation is not None:
@@ -247,11 +252,11 @@ class _Bulk:
         replacement: Mapping[str, Any],
         upsert: bool = False,
         collation: Optional[Mapping[str, Any]] = None,
-        hint: Union[str, SON[str, Any], None] = None,
+        hint: Union[str, dict[str, Any], None] = None,
     ) -> None:
         """Create a replace document and add it to the list of ops."""
         validate_ok_for_replace(replacement)
-        cmd = SON([("q", selector), ("u", replacement), ("multi", False), ("upsert", upsert)])
+        cmd = {"q": selector, "u": replacement, "multi": False, "upsert": upsert}
         if collation is not None:
             self.uses_collation = True
             cmd["collation"] = collation
@@ -265,10 +270,10 @@ class _Bulk:
         selector: Mapping[str, Any],
         limit: int,
         collation: Optional[Mapping[str, Any]] = None,
-        hint: Union[str, SON[str, Any], None] = None,
+        hint: Union[str, dict[str, Any], None] = None,
     ) -> None:
         """Create a delete document and add it to the list of ops."""
-        cmd = SON([("q", selector), ("limit", limit)])
+        cmd = {"q": selector, "limit": limit}
         if collation is not None:
             self.uses_collation = True
             cmd["collation"] = collation
@@ -355,7 +360,7 @@ class _Bulk:
                 if last_run and (len(run.ops) - run.idx_offset) == 1:
                     write_concern = final_write_concern or write_concern
 
-                cmd = SON([(cmd_name, self.collection.name), ("ordered", self.ordered)])
+                cmd = {cmd_name: self.collection.name, "ordered": self.ordered}
                 if self.comment:
                     cmd["comment"] = self.comment
                 _csot.apply_write_concern(cmd, write_concern)
@@ -414,7 +419,8 @@ class _Bulk:
         generator: Iterator[Any],
         write_concern: WriteConcern,
         session: Optional[ClientSession],
-    ) -> Dict[str, Any]:
+        operation: str,
+    ) -> dict[str, Any]:
         """Execute using write commands."""
         # nModified is only reported for write commands, not legacy ops.
         full_result = {
@@ -443,8 +449,14 @@ class _Bulk:
             )
 
         client = self.collection.database.client
-        with client._tmp_session(session) as s:
-            client._retry_with_session(self.is_retryable, retryable_bulk, s, self)
+        client._retryable_write(
+            self.is_retryable,
+            retryable_bulk,
+            session,
+            operation,
+            bulk=self,
+            operation_id=op_id,
+        )
 
         if full_result["writeErrors"] or full_result["writeConcernErrors"]:
             _raise_bulk_write_error(full_result)
@@ -475,13 +487,11 @@ class _Bulk:
             )
 
             while run.idx_offset < len(run.ops):
-                cmd = SON(
-                    [
-                        (cmd_name, self.collection.name),
-                        ("ordered", False),
-                        ("writeConcern", {"w": 0}),
-                    ]
-                )
+                cmd = {
+                    cmd_name: self.collection.name,
+                    "ordered": False,
+                    "writeConcern": {"w": 0},
+                }
                 conn.add_server_api(cmd)
                 ops = islice(run.ops, run.idx_offset, None)
                 # Run as many ops as possible.
@@ -556,7 +566,12 @@ class _Bulk:
             return self.execute_command_no_results(conn, generator, write_concern)
         return self.execute_op_msg_no_results(conn, generator)
 
-    def execute(self, write_concern: WriteConcern, session: Optional[ClientSession]) -> Any:
+    def execute(
+        self,
+        write_concern: WriteConcern,
+        session: Optional[ClientSession],
+        operation: str,
+    ) -> Any:
         """Execute operations."""
         if not self.ops:
             raise InvalidOperation("No operations to execute")
@@ -573,8 +588,8 @@ class _Bulk:
 
         client = self.collection.database.client
         if not write_concern.acknowledged:
-            with client._conn_for_writes(session) as connection:
+            with client._conn_for_writes(session, operation) as connection:
                 self.execute_no_results(connection, generator, write_concern)
                 return None
         else:
-            return self.execute_command(generator, write_concern, session)
+            return self.execute_command(generator, write_concern, session, operation)
