@@ -16,9 +16,12 @@
 
 .. _collations: https://www.mongodb.com/docs/manual/reference/collation/
 """
-from typing import Any, Dict, Mapping, Optional, Union
+from __future__ import annotations
+
+from typing import Any, Mapping, Optional, Union
 
 from pymongo import common
+from pymongo.write_concern import validate_boolean
 
 
 class CollationStrength:
@@ -94,22 +97,21 @@ class CollationCaseFirst:
 class Collation:
     """Collation
 
-    :Parameters:
-      - `locale`: (string) The locale of the collation. This should be a string
+    :param locale: (string) The locale of the collation. This should be a string
         that identifies an `ICU locale ID` exactly. For example, ``en_US`` is
         valid, but ``en_us`` and ``en-US`` are not. Consult the MongoDB
         documentation for a list of supported locales.
-      - `caseLevel`: (optional) If ``True``, turn on case sensitivity if
+    :param caseLevel: (optional) If ``True``, turn on case sensitivity if
         `strength` is 1 or 2 (case sensitivity is implied if `strength` is
         greater than 2). Defaults to ``False``.
-      - `caseFirst`: (optional) Specify that either uppercase or lowercase
+    :param caseFirst: (optional) Specify that either uppercase or lowercase
         characters take precedence. Must be one of the following values:
 
           * :data:`~CollationCaseFirst.UPPER`
           * :data:`~CollationCaseFirst.LOWER`
           * :data:`~CollationCaseFirst.OFF` (the default)
 
-      - `strength`: (optional) Specify the comparison strength. This is also
+    :param strength: Specify the comparison strength. This is also
         known as the ICU comparison level. This must be one of the following
         values:
 
@@ -123,27 +125,27 @@ class Collation:
         `strength` of :data:`~CollationStrength.SECONDARY` differentiates
         characters based both on the unadorned base character and its accents.
 
-      - `numericOrdering`: (optional) If ``True``, order numbers numerically
+    :param numericOrdering: If ``True``, order numbers numerically
         instead of in collation order (defaults to ``False``).
-      - `alternate`: (optional) Specify whether spaces and punctuation are
+    :param alternate: Specify whether spaces and punctuation are
         considered base characters. This must be one of the following values:
 
           * :data:`~CollationAlternate.NON_IGNORABLE` (the default)
           * :data:`~CollationAlternate.SHIFTED`
 
-      - `maxVariable`: (optional) When `alternate` is
+    :param maxVariable: When `alternate` is
         :data:`~CollationAlternate.SHIFTED`, this option specifies what
         characters may be ignored. This must be one of the following values:
 
           * :data:`~CollationMaxVariable.PUNCT` (the default)
           * :data:`~CollationMaxVariable.SPACE`
 
-      - `normalization`: (optional) If ``True``, normalizes text into Unicode
+    :param normalization: If ``True``, normalizes text into Unicode
         NFD. Defaults to ``False``.
-      - `backwards`: (optional) If ``True``, accents on characters are
+    :param backwards: If ``True``, accents on characters are
         considered from the back of the word to the front, as it is done in some
         French dictionary ordering traditions. Defaults to ``False``.
-      - `kwargs`: (optional) Keyword arguments supplying any additional options
+    :param kwargs: Keyword arguments supplying any additional options
         to be sent with this Collation object.
 
     .. versionadded: 3.4
@@ -166,15 +168,15 @@ class Collation:
         **kwargs: Any,
     ) -> None:
         locale = common.validate_string("locale", locale)
-        self.__document: Dict[str, Any] = {"locale": locale}
+        self.__document: dict[str, Any] = {"locale": locale}
         if caseLevel is not None:
-            self.__document["caseLevel"] = common.validate_boolean("caseLevel", caseLevel)
+            self.__document["caseLevel"] = validate_boolean("caseLevel", caseLevel)
         if caseFirst is not None:
             self.__document["caseFirst"] = common.validate_string("caseFirst", caseFirst)
         if strength is not None:
             self.__document["strength"] = common.validate_integer("strength", strength)
         if numericOrdering is not None:
-            self.__document["numericOrdering"] = common.validate_boolean(
+            self.__document["numericOrdering"] = validate_boolean(
                 "numericOrdering", numericOrdering
             )
         if alternate is not None:
@@ -182,15 +184,13 @@ class Collation:
         if maxVariable is not None:
             self.__document["maxVariable"] = common.validate_string("maxVariable", maxVariable)
         if normalization is not None:
-            self.__document["normalization"] = common.validate_boolean(
-                "normalization", normalization
-            )
+            self.__document["normalization"] = validate_boolean("normalization", normalization)
         if backwards is not None:
-            self.__document["backwards"] = common.validate_boolean("backwards", backwards)
+            self.__document["backwards"] = validate_boolean("backwards", backwards)
         self.__document.update(kwargs)
 
     @property
-    def document(self) -> Dict[str, Any]:
+    def document(self) -> dict[str, Any]:
         """The document representation of this collation.
 
         .. note::
@@ -214,7 +214,7 @@ class Collation:
 
 def validate_collation_or_none(
     value: Optional[Union[Mapping[str, Any], Collation]]
-) -> Optional[Dict[str, Any]]:
+) -> Optional[dict[str, Any]]:
     if value is None:
         return None
     if isinstance(value, Collation):
