@@ -14,40 +14,44 @@
 from __future__ import annotations
 
 import warnings
-from typing import Any, Iterable, List, Optional, Union
-
-try:
-    import snappy
-
-    _HAVE_SNAPPY = True
-except ImportError:
-    # python-snappy isn't available.
-    _HAVE_SNAPPY = False
-
-try:
-    import zlib
-
-    _HAVE_ZLIB = True
-except ImportError:
-    # Python built without zlib support.
-    _HAVE_ZLIB = False
-
-try:
-    from zstandard import ZstdCompressor, ZstdDecompressor
-
-    _HAVE_ZSTD = True
-except ImportError:
-    _HAVE_ZSTD = False
+from typing import Any, Iterable, Optional, Union
 
 from pymongo.hello import HelloCompat
-from pymongo.monitoring import _SENSITIVE_COMMANDS
+from pymongo.helpers import _SENSITIVE_COMMANDS
 
 _SUPPORTED_COMPRESSORS = {"snappy", "zlib", "zstd"}
 _NO_COMPRESSION = {HelloCompat.CMD, HelloCompat.LEGACY_CMD}
 _NO_COMPRESSION.update(_SENSITIVE_COMMANDS)
 
 
-def validate_compressors(dummy: Any, value: Union[str, Iterable[str]]) -> List[str]:
+def _have_snappy() -> bool:
+    try:
+        import snappy  # type:ignore[import]  # noqa: F401
+
+        return True
+    except ImportError:
+        return False
+
+
+def _have_zlib() -> bool:
+    try:
+        import zlib  # noqa: F401
+
+        return True
+    except ImportError:
+        return False
+
+
+def _have_zstd() -> bool:
+    try:
+        import zstandard  # noqa: F401
+
+        return True
+    except ImportError:
+        return False
+
+
+def validate_compressors(dummy: Any, value: Union[str, Iterable[str]]) -> list[str]:
     try:
         # `value` is string.
         compressors = value.split(",")  # type: ignore[union-attr]
@@ -58,24 +62,27 @@ def validate_compressors(dummy: Any, value: Union[str, Iterable[str]]) -> List[s
     for compressor in compressors[:]:
         if compressor not in _SUPPORTED_COMPRESSORS:
             compressors.remove(compressor)
-            warnings.warn(f"Unsupported compressor: {compressor}")
-        elif compressor == "snappy" and not _HAVE_SNAPPY:
+            warnings.warn(f"Unsupported compressor: {compressor}", stacklevel=2)
+        elif compressor == "snappy" and not _have_snappy():
             compressors.remove(compressor)
             warnings.warn(
                 "Wire protocol compression with snappy is not available. "
-                "You must install the python-snappy module for snappy support."
+                "You must install the python-snappy module for snappy support.",
+                stacklevel=2,
             )
-        elif compressor == "zlib" and not _HAVE_ZLIB:
+        elif compressor == "zlib" and not _have_zlib():
             compressors.remove(compressor)
             warnings.warn(
                 "Wire protocol compression with zlib is not available. "
-                "The zlib module is not available."
+                "The zlib module is not available.",
+                stacklevel=2,
             )
-        elif compressor == "zstd" and not _HAVE_ZSTD:
+        elif compressor == "zstd" and not _have_zstd():
             compressors.remove(compressor)
             warnings.warn(
                 "Wire protocol compression with zstandard is not available. "
-                "You must install the zstandard module for zstandard support."
+                "You must install the zstandard module for zstandard support.",
+                stacklevel=2,
             )
     return compressors
 
@@ -84,19 +91,19 @@ def validate_zlib_compression_level(option: str, value: Any) -> int:
     try:
         level = int(value)
     except Exception:
-        raise TypeError(f"{option} must be an integer, not {value!r}.")
+        raise TypeError(f"{option} must be an integer, not {value!r}.") from None
     if level < -1 or level > 9:
         raise ValueError("%s must be between -1 and 9, not %d." % (option, level))
     return level
 
 
 class CompressionSettings:
-    def __init__(self, compressors: List[str], zlib_compression_level: int):
+    def __init__(self, compressors: list[str], zlib_compression_level: int):
         self.compressors = compressors
         self.zlib_compression_level = zlib_compression_level
 
     def get_compression_context(
-        self, compressors: Optional[List[str]]
+        self, compressors: Optional[list[str]]
     ) -> Union[SnappyContext, ZlibContext, ZstdContext, None]:
         if compressors:
             chosen = compressors[0]
@@ -115,6 +122,8 @@ class SnappyContext:
 
     @staticmethod
     def compress(data: bytes) -> bytes:
+        import snappy
+
         return snappy.compress(data)
 
 
@@ -125,6 +134,8 @@ class ZlibContext:
         self.level = level
 
     def compress(self, data: bytes) -> bytes:
+        import zlib
+
         return zlib.compress(data, self.level)
 
 
@@ -135,7 +146,9 @@ class ZstdContext:
     def compress(data: bytes) -> bytes:
         # ZstdCompressor is not thread safe.
         # TODO: Use a pool?
-        return ZstdCompressor().compress(data)
+        import zstandard
+
+        return zstandard.ZstdCompressor().compress(data)
 
 
 def decompress(data: bytes, compressor_id: int) -> bytes:
@@ -144,12 +157,18 @@ def decompress(data: bytes, compressor_id: int) -> bytes:
         # https://github.com/andrix/python-snappy/issues/65
         # This only matters when data is a memoryview since
         # id(bytes(data)) == id(data) when data is a bytes.
+        import snappy
+
         return snappy.uncompress(bytes(data))
     elif compressor_id == ZlibContext.compressor_id:
+        import zlib
+
         return zlib.decompress(data)
     elif compressor_id == ZstdContext.compressor_id:
         # ZstdDecompressor is not thread safe.
         # TODO: Use a pool?
-        return ZstdDecompressor().decompress(data)
+        import zstandard
+
+        return zstandard.ZstdDecompressor().decompress(data)
     else:
         raise ValueError("Unknown compressorId %d" % (compressor_id,))
