@@ -18,6 +18,7 @@ from __future__ import annotations
 import contextlib
 import enum
 import socket
+import uuid
 import weakref
 from copy import deepcopy
 from typing import (
@@ -30,15 +31,16 @@ from typing import (
     MutableMapping,
     Optional,
     Sequence,
-    Tuple,
+    Union,
+    cast,
 )
 
 try:
-    from pymongocrypt.auto_encrypter import AutoEncrypter
-    from pymongocrypt.errors import MongoCryptError  # noqa: F401
-    from pymongocrypt.explicit_encrypter import ExplicitEncrypter
-    from pymongocrypt.mongocrypt import MongoCryptOptions
-    from pymongocrypt.state_machine import MongoCryptCallback
+    from pymongocrypt.auto_encrypter import AutoEncrypter  # type:ignore[import]
+    from pymongocrypt.errors import MongoCryptError  # type:ignore[import]
+    from pymongocrypt.explicit_encrypter import ExplicitEncrypter  # type:ignore[import]
+    from pymongocrypt.mongocrypt import MongoCryptOptions  # type:ignore[import]
+    from pymongocrypt.state_machine import MongoCryptCallback  # type:ignore[import]
 
     _HAVE_PYMONGOCRYPT = True
 except ImportError:
@@ -50,7 +52,6 @@ from bson.binary import STANDARD, UUID_SUBTYPE, Binary
 from bson.codec_options import CodecOptions
 from bson.errors import BSONError
 from bson.raw_bson import DEFAULT_RAW_BSON_OPTIONS, RawBSONDocument, _inflate_bson
-from bson.son import SON
 from pymongo import _csot
 from pymongo.collection import Collection
 from pymongo.common import CONNECT_TIMEOUT
@@ -73,7 +74,7 @@ from pymongo.pool import PoolOptions, _configured_socket, _raise_connection_fail
 from pymongo.read_concern import ReadConcern
 from pymongo.results import BulkWriteResult, DeleteResult
 from pymongo.ssl_support import get_ssl_context
-from pymongo.typings import _DocumentType
+from pymongo.typings import _DocumentType, _DocumentTypeArg
 from pymongo.uri_parser import parse_host
 from pymongo.write_concern import WriteConcern
 
@@ -84,8 +85,9 @@ _HTTPS_PORT = 443
 _KMS_CONNECT_TIMEOUT = CONNECT_TIMEOUT  # CDRIVER-3262 redefined this value to CONNECT_TIMEOUT
 _MONGOCRYPTD_TIMEOUT_MS = 10000
 
-
-_DATA_KEY_OPTS: CodecOptions = CodecOptions(document_class=SON, uuid_representation=STANDARD)
+_DATA_KEY_OPTS: CodecOptions[dict[str, Any]] = CodecOptions(
+    document_class=Dict[str, Any], uuid_representation=STANDARD
+)
 # Use RawBSONDocument codec options to avoid needlessly decoding
 # documents from the key vault.
 _KEY_VAULT_OPTS = CodecOptions(document_class=RawBSONDocument)
@@ -101,15 +103,15 @@ def _wrap_encryption_errors() -> Iterator[None]:
         # we should propagate them unchanged.
         raise
     except Exception as exc:
-        raise EncryptionError(exc)
+        raise EncryptionError(exc) from exc
 
 
 class _EncryptionIO(MongoCryptCallback):  # type: ignore[misc]
     def __init__(
         self,
-        client: Optional[MongoClient],
-        key_vault_coll: Collection,
-        mongocryptd_client: Optional[MongoClient],
+        client: Optional[MongoClient[_DocumentTypeArg]],
+        key_vault_coll: Collection[_DocumentTypeArg],
+        mongocryptd_client: Optional[MongoClient[_DocumentTypeArg]],
         opts: AutoEncryptionOpts,
     ):
         """Internal class to perform I/O on behalf of pymongocrypt."""
@@ -119,10 +121,13 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore[misc]
             self.client_ref = weakref.ref(client)
         else:
             self.client_ref = None
-        self.key_vault_coll: Optional[Collection] = key_vault_coll.with_options(
-            codec_options=_KEY_VAULT_OPTS,
-            read_concern=ReadConcern(level="majority"),
-            write_concern=WriteConcern(w="majority"),
+        self.key_vault_coll: Optional[Collection[RawBSONDocument]] = cast(
+            Collection[RawBSONDocument],
+            key_vault_coll.with_options(
+                codec_options=_KEY_VAULT_OPTS,
+                read_concern=ReadConcern(level="majority"),
+                write_concern=WriteConcern(w="majority"),
+            ),
         )
         self.mongocryptd_client = mongocryptd_client
         self.opts = opts
@@ -131,11 +136,9 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore[misc]
     def kms_request(self, kms_context: MongoCryptKmsContext) -> None:
         """Complete a KMS request.
 
-        :Parameters:
-          - `kms_context`: A :class:`MongoCryptKmsContext`.
+        :param kms_context: A :class:`MongoCryptKmsContext`.
 
-        :Returns:
-          None
+        :return: None
         """
         endpoint = kms_context.endpoint
         message = kms_context.message
@@ -173,7 +176,7 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore[misc]
                         raise OSError("KMS connection closed")
                     kms_context.feed(data)
             except BLOCKING_IO_ERRORS:
-                raise socket.timeout("timed out")
+                raise socket.timeout("timed out") from None
             finally:
                 conn.close()
         except (PyMongoError, MongoCryptError):
@@ -182,18 +185,18 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore[misc]
             # Wrap I/O errors in PyMongo exceptions.
             _raise_connection_failure((host, port), error)
 
-    def collection_info(self, database: Database, filter: bytes) -> Optional[bytes]:
+    def collection_info(
+        self, database: Database[Mapping[str, Any]], filter: bytes
+    ) -> Optional[bytes]:
         """Get the collection info for a namespace.
 
         The returned collection info is passed to libmongocrypt which reads
         the JSON schema.
 
-        :Parameters:
-          - `database`: The database on which to run listCollections.
-          - `filter`: The filter to pass to listCollections.
+        :param database: The database on which to run listCollections.
+        :param filter: The filter to pass to listCollections.
 
-        :Returns:
-          The first document from the listCollections command response as BSON.
+        :return: The first document from the listCollections command response as BSON.
         """
         with self.client_ref()[database].list_collections(filter=RawBSONDocument(filter)) as cursor:
             for doc in cursor:
@@ -214,12 +217,10 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore[misc]
     def mark_command(self, database: str, cmd: bytes) -> bytes:
         """Mark a command for encryption.
 
-        :Parameters:
-          - `database`: The database on which to run this command.
-          - `cmd`: The BSON command to run.
+        :param database: The database on which to run this command.
+        :param cmd: The BSON command to run.
 
-        :Returns:
-          The marked command response from mongocryptd.
+        :return: The marked command response from mongocryptd.
         """
         if not self._spawned and not self.opts._mongocryptd_bypass_spawn:
             self.spawn()
@@ -243,11 +244,9 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore[misc]
     def fetch_keys(self, filter: bytes) -> Iterator[bytes]:
         """Yields one or more keys from the key vault.
 
-        :Parameters:
-          - `filter`: The filter to pass to find.
+        :param filter: The filter to pass to find.
 
-        :Returns:
-          A generator which yields the requested keys from the key vault.
+        :return: A generator which yields the requested keys from the key vault.
         """
         assert self.key_vault_coll is not None
         with self.key_vault_coll.find(RawBSONDocument(filter)) as cursor:
@@ -257,11 +256,9 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore[misc]
     def insert_data_key(self, data_key: bytes) -> Binary:
         """Insert a data key into the key vault.
 
-        :Parameters:
-          - `data_key`: The data key document to insert.
+        :param data_key: The data key document to insert.
 
-        :Returns:
-          The _id of the inserted data key document.
+        :return: The _id of the inserted data key document.
         """
         raw_doc = RawBSONDocument(data_key, _KEY_VAULT_OPTS)
         data_key_id = raw_doc.get("_id")
@@ -277,11 +274,9 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore[misc]
 
         A document can be any mapping type (like :class:`dict`).
 
-        :Parameters:
-          - `doc`: mapping type representing a document
+        :param doc: mapping type representing a document
 
-        :Returns:
-          The encoded BSON bytes.
+        :return: The encoded BSON bytes.
         """
         return encode(doc)
 
@@ -316,6 +311,9 @@ class RewrapManyDataKeyResult:
         """
         return self._bulk_write_result
 
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}({self._bulk_write_result!r})"
+
 
 class _Encrypter:
     """Encrypts and decrypts MongoDB commands.
@@ -324,12 +322,11 @@ class _Encrypter:
     MongoDB commands.
     """
 
-    def __init__(self, client: MongoClient, opts: AutoEncryptionOpts):
+    def __init__(self, client: MongoClient[_DocumentTypeArg], opts: AutoEncryptionOpts):
         """Create a _Encrypter for a client.
 
-        :Parameters:
-          - `client`: The encrypted MongoClient.
-          - `opts`: The encrypted client's :class:`AutoEncryptionOpts`.
+        :param client: The encrypted MongoClient.
+        :param opts: The encrypted client's :class:`AutoEncryptionOpts`.
         """
         if opts._schema_map is None:
             schema_map = None
@@ -343,7 +340,9 @@ class _Encrypter:
         self._bypass_auto_encryption = opts._bypass_auto_encryption
         self._internal_client = None
 
-        def _get_internal_client(encrypter: _Encrypter, mongo_client: MongoClient) -> MongoClient:
+        def _get_internal_client(
+            encrypter: _Encrypter, mongo_client: MongoClient[_DocumentTypeArg]
+        ) -> MongoClient[_DocumentTypeArg]:
             if mongo_client.options.pool_options.max_pool_size is None:
                 # Unlimited pool size, use the same client.
                 return mongo_client
@@ -367,11 +366,13 @@ class _Encrypter:
         db, coll = opts._key_vault_namespace.split(".", 1)
         key_vault_coll = key_vault_client[db][coll]
 
-        mongocryptd_client: MongoClient = MongoClient(
+        mongocryptd_client: MongoClient[Mapping[str, Any]] = MongoClient(
             opts._mongocryptd_uri, connect=False, serverSelectionTimeoutMS=_MONGOCRYPTD_TIMEOUT_MS
         )
 
-        io_callbacks = _EncryptionIO(metadata_client, key_vault_coll, mongocryptd_client, opts)
+        io_callbacks = _EncryptionIO(  # type:ignore[misc]
+            metadata_client, key_vault_coll, mongocryptd_client, opts
+        )
         self._auto_encrypter = AutoEncrypter(
             io_callbacks,
             MongoCryptOptions(
@@ -387,38 +388,33 @@ class _Encrypter:
         self._closed = False
 
     def encrypt(
-        self, database: str, cmd: Mapping[str, Any], codec_options: CodecOptions
-    ) -> Dict[Any, Any]:
+        self, database: str, cmd: Mapping[str, Any], codec_options: CodecOptions[_DocumentTypeArg]
+    ) -> dict[str, Any]:
         """Encrypt a MongoDB command.
 
-        :Parameters:
-          - `database`: The database for this command.
-          - `cmd`: A command document.
-          - `codec_options`: The CodecOptions to use while encoding `cmd`.
+        :param database: The database for this command.
+        :param cmd: A command document.
+        :param codec_options: The CodecOptions to use while encoding `cmd`.
 
-        :Returns:
-          The encrypted command to execute.
+        :return: The encrypted command to execute.
         """
         self._check_closed()
         encoded_cmd = _dict_to_bson(cmd, False, codec_options)
         with _wrap_encryption_errors():
             encrypted_cmd = self._auto_encrypter.encrypt(database, encoded_cmd)
             # TODO: PYTHON-1922 avoid decoding the encrypted_cmd.
-            encrypt_cmd = _inflate_bson(encrypted_cmd, DEFAULT_RAW_BSON_OPTIONS)
-            return encrypt_cmd
+            return _inflate_bson(encrypted_cmd, DEFAULT_RAW_BSON_OPTIONS)
 
     def decrypt(self, response: bytes) -> Optional[bytes]:
         """Decrypt a MongoDB command response.
 
-        :Parameters:
-          - `response`: A MongoDB command response as BSON.
+        :param response: A MongoDB command response as BSON.
 
-        :Returns:
-          The decrypted command response.
+        :return: The decrypted command response.
         """
         self._check_closed()
         with _wrap_encryption_errors():
-            return self._auto_encrypter.decrypt(response)
+            return cast(bytes, self._auto_encrypter.decrypt(response))
 
     def _check_closed(self) -> None:
         if self._closed:
@@ -484,8 +480,8 @@ class ClientEncryption(Generic[_DocumentType]):
         self,
         kms_providers: Mapping[str, Any],
         key_vault_namespace: str,
-        key_vault_client: MongoClient,
-        codec_options: CodecOptions,
+        key_vault_client: MongoClient[_DocumentTypeArg],
+        codec_options: CodecOptions[_DocumentTypeArg],
         kms_tls_options: Optional[Mapping[str, Any]] = None,
     ) -> None:
         """Explicit client-side field level encryption.
@@ -501,8 +497,7 @@ class ClientEncryption(Generic[_DocumentType]):
 
         See :ref:`explicit-client-side-encryption` for an example.
 
-        :Parameters:
-          - `kms_providers`: Map of KMS provider options. The `kms_providers`
+        :param kms_providers: Map of KMS provider options. The `kms_providers`
             map values differ by provider:
 
               - `aws`: Map with "accessKeyId" and "secretAccessKey" as strings.
@@ -527,20 +522,23 @@ class ClientEncryption(Generic[_DocumentType]):
                 data keys. This key should be generated and stored as securely
                 as possible.
 
-          - `key_vault_namespace`: The namespace for the key vault collection.
+            KMS providers may be specified with an optional name suffix
+            separated by a colon, for example "kmip:name" or "aws:name".
+            Named KMS providers do not support :ref:`CSFLE on-demand credentials`.
+        :param key_vault_namespace: The namespace for the key vault collection.
             The key vault collection contains all data keys used for encryption
             and decryption. Data keys are stored as documents in this MongoDB
             collection. Data keys are protected with encryption by a KMS
             provider.
-          - `key_vault_client`: A MongoClient connected to a MongoDB cluster
+        :param key_vault_client: A MongoClient connected to a MongoDB cluster
             containing the `key_vault_namespace` collection.
-          - `codec_options`: An instance of
+        :param codec_options: An instance of
             :class:`~bson.codec_options.CodecOptions` to use when encoding a
             value for encryption and decoding the decrypted BSON value. This
             should be the same CodecOptions instance configured on the
             MongoClient, Database, or Collection used to access application
             data.
-          - `kms_tls_options` (optional): A map of KMS provider names to TLS
+        :param kms_tls_options: A map of KMS provider names to TLS
             options to use when creating secure connections to KMS providers.
             Accepts the same TLS options as
             :class:`pymongo.mongo_client.MongoClient`. For example, to
@@ -585,17 +583,18 @@ class ClientEncryption(Generic[_DocumentType]):
             self._io_callbacks, MongoCryptOptions(kms_providers, None)
         )
         # Use the same key vault collection as the callback.
+        assert self._io_callbacks.key_vault_coll is not None
         self._key_vault_coll = self._io_callbacks.key_vault_coll
 
     def create_encrypted_collection(
         self,
-        database: Database,
+        database: Database[_DocumentTypeArg],
         name: str,
         encrypted_fields: Mapping[str, Any],
         kms_provider: Optional[str] = None,
         master_key: Optional[Mapping[str, Any]] = None,
         **kwargs: Any,
-    ) -> Tuple[Collection[_DocumentType], Mapping[str, Any]]:
+    ) -> tuple[Collection[_DocumentTypeArg], Mapping[str, Any]]:
         """Create a collection with encryptedFields.
 
         .. warning::
@@ -608,10 +607,11 @@ class ClientEncryption(Generic[_DocumentType]):
         creation. :class:`~pymongo.errors.EncryptionError` will be
         raised if the collection already exists.
 
-        :Parameters:
-          - `name`: the name of the collection to create
-          - `encrypted_fields` (dict): Document that describes the encrypted fields for
-            Queryable Encryption. For example::
+        :param name: the name of the collection to create
+        :param encrypted_fields: Document that describes the encrypted fields for
+            Queryable Encryption. The "keyId" may be set to ``None`` to auto-generate the data keys.  For example:
+
+            .. code-block: python
 
               {
                 "escCollection": "enxcol_.encryptedCollection.esc",
@@ -631,19 +631,17 @@ class ClientEncryption(Generic[_DocumentType]):
                   ]
               }
 
-            The "keyId" may be set to ``None`` to auto-generate the data keys.
-          - `kms_provider` (optional): the KMS provider to be used
-          - `master_key` (optional): Identifies a KMS-specific key used to encrypt the
+          :param kms_provider: the KMS provider to be used
+          :param master_key: Identifies a KMS-specific key used to encrypt the
             new data key. If the kmsProvider is "local" the `master_key` is
             not applicable and may be omitted.
-          - `**kwargs` (optional): additional keyword arguments are the same as "create_collection".
+          :param kwargs: additional keyword arguments are the same as "create_collection".
 
         All optional `create collection command`_ parameters should be passed
         as keyword arguments to this method.
         See the documentation for :meth:`~pymongo.database.Database.create_collection` for all valid options.
 
-        :Raises:
-          - :class:`~pymongo.errors.EncryptedCollectionError`: When either data-key creation or creating the collection fails.
+        :raises: - :class:`~pymongo.errors.EncryptedCollectionError`: When either data-key creation or creating the collection fails.
 
         .. versionadded:: 4.4
 
@@ -680,14 +678,14 @@ class ClientEncryption(Generic[_DocumentType]):
     ) -> Binary:
         """Create and insert a new data key into the key vault collection.
 
-        :Parameters:
-          - `kms_provider`: The KMS provider to use. Supported values are
-            "aws", "azure", "gcp", "kmip", and "local".
-          - `master_key`: Identifies a KMS-specific key used to encrypt the
+        :param kms_provider: The KMS provider to use. Supported values are
+            "aws", "azure", "gcp", "kmip", "local", or a named provider like
+            "kmip:name".
+        :param master_key: Identifies a KMS-specific key used to encrypt the
             new data key. If the kmsProvider is "local" the `master_key` is
             not applicable and may be omitted.
 
-            If the `kms_provider` is "aws" it is required and has the
+            If the `kms_provider` type is "aws" it is required and has the
             following fields::
 
               - `region` (string): Required. The AWS region, e.g. "us-east-1".
@@ -697,7 +695,7 @@ class ClientEncryption(Generic[_DocumentType]):
                 requests to. May include port number, e.g.
                 "kms.us-east-1.amazonaws.com:443".
 
-            If the `kms_provider` is "azure" it is required and has the
+            If the `kms_provider` type is "azure" it is required and has the
             following fields::
 
               - `keyVaultEndpoint` (string): Required. Host with optional
@@ -705,7 +703,7 @@ class ClientEncryption(Generic[_DocumentType]):
               - `keyName` (string): Required. Key name in the key vault.
               - `keyVersion` (string): Optional. Version of the key to use.
 
-            If the `kms_provider` is "gcp" it is required and has the
+            If the `kms_provider` type is "gcp" it is required and has the
             following fields::
 
               - `projectId` (string): Required. The Google cloud project ID.
@@ -717,7 +715,7 @@ class ClientEncryption(Generic[_DocumentType]):
               - `endpoint` (string): Optional. Host with optional port.
                 Defaults to "cloudkms.googleapis.com".
 
-            If the `kms_provider` is "kmip" it is optional and has the
+            If the `kms_provider` type is "kmip" it is optional and has the
             following fields::
 
               - `keyId` (string): Optional. `keyId` is the KMIP Unique
@@ -727,7 +725,7 @@ class ClientEncryption(Generic[_DocumentType]):
               - `endpoint` (string): Optional. Host with optional
                  port, e.g. "example.vault.azure.net:".
 
-          - `key_alt_names` (optional): An optional list of string alternate
+        :param key_alt_names: An optional list of string alternate
             names used to reference a key. If a key is created with alternate
             names, then encryption may refer to the key by the unique alternate
             name instead of by ``key_id``. The following example shows creating
@@ -737,11 +735,10 @@ class ClientEncryption(Generic[_DocumentType]):
               # reference the key with the alternate name
               client_encryption.encrypt("457-55-5462", key_alt_name="name1",
                                         algorithm=Algorithm.AEAD_AES_256_CBC_HMAC_SHA_512_Random)
-          - `key_material` (optional): Sets the custom key material to be used
+        :param key_material: Sets the custom key material to be used
             by the data key for encryption and decryption.
 
-        :Returns:
-          The ``_id`` of the created data key document as a
+        :return: The ``_id`` of the created data key document as a
           :class:`~bson.binary.Binary` with subtype
           :data:`~bson.binary.UUID_SUBTYPE`.
 
@@ -750,18 +747,21 @@ class ClientEncryption(Generic[_DocumentType]):
         """
         self._check_closed()
         with _wrap_encryption_errors():
-            return self._encryption.create_data_key(
-                kms_provider,
-                master_key=master_key,
-                key_alt_names=key_alt_names,
-                key_material=key_material,
+            return cast(
+                Binary,
+                self._encryption.create_data_key(
+                    kms_provider,
+                    master_key=master_key,
+                    key_alt_names=key_alt_names,
+                    key_material=key_material,
+                ),
             )
 
     def _encrypt_helper(
         self,
         value: Any,
         algorithm: str,
-        key_id: Optional[Binary] = None,
+        key_id: Optional[Union[Binary, uuid.UUID]] = None,
         key_alt_name: Optional[str] = None,
         query_type: Optional[str] = None,
         contention_factor: Optional[int] = None,
@@ -769,6 +769,8 @@ class ClientEncryption(Generic[_DocumentType]):
         is_expression: bool = False,
     ) -> Any:
         self._check_closed()
+        if isinstance(key_id, uuid.UUID):
+            key_id = Binary.from_uuid(key_id)
         if key_id is not None and not (
             isinstance(key_id, Binary) and key_id.subtype == UUID_SUBTYPE
         ):
@@ -801,7 +803,7 @@ class ClientEncryption(Generic[_DocumentType]):
         self,
         value: Any,
         algorithm: str,
-        key_id: Optional[Binary] = None,
+        key_id: Optional[Union[Binary, uuid.UUID]] = None,
         key_alt_name: Optional[str] = None,
         query_type: Optional[str] = None,
         contention_factor: Optional[int] = None,
@@ -812,43 +814,47 @@ class ClientEncryption(Generic[_DocumentType]):
         Note that exactly one of ``key_id`` or  ``key_alt_name`` must be
         provided.
 
-        :Parameters:
-          - `value`: The BSON value to encrypt.
-          - `algorithm` (string): The encryption algorithm to use. See
+        :param value: The BSON value to encrypt.
+        :param algorithm` (string): The encryption algorithm to use. See
             :class:`Algorithm` for some valid options.
-          - `key_id`: Identifies a data key by ``_id`` which must be a
+        :param key_id: Identifies a data key by ``_id`` which must be a
             :class:`~bson.binary.Binary` with subtype 4 (
             :attr:`~bson.binary.UUID_SUBTYPE`).
-          - `key_alt_name`: Identifies a key vault document by 'keyAltName'.
-          - `query_type` (str): The query type to execute. See :class:`QueryType` for valid options.
-          - `contention_factor` (int): The contention factor to use
+        :param key_alt_name: Identifies a key vault document by 'keyAltName'.
+        :param query_type` (str): The query type to execute. See :class:`QueryType` for valid options.
+        :param contention_factor` (int): The contention factor to use
             when the algorithm is :attr:`Algorithm.INDEXED`.  An integer value
             *must* be given when the :attr:`Algorithm.INDEXED` algorithm is
             used.
-          - `range_opts`: Experimental only, not intended for public use.
+        :param range_opts: Experimental only, not intended for public use.
 
-        :Returns:
-          The encrypted value, a :class:`~bson.binary.Binary` with subtype 6.
+        :return: The encrypted value, a :class:`~bson.binary.Binary` with subtype 6.
+
+        .. versionchanged:: 4.7
+           ``key_id`` can now be passed in as a :class:`uuid.UUID`.
 
         .. versionchanged:: 4.2
            Added the `query_type` and `contention_factor` parameters.
         """
-        return self._encrypt_helper(
-            value=value,
-            algorithm=algorithm,
-            key_id=key_id,
-            key_alt_name=key_alt_name,
-            query_type=query_type,
-            contention_factor=contention_factor,
-            range_opts=range_opts,
-            is_expression=False,
+        return cast(
+            Binary,
+            self._encrypt_helper(
+                value=value,
+                algorithm=algorithm,
+                key_id=key_id,
+                key_alt_name=key_alt_name,
+                query_type=query_type,
+                contention_factor=contention_factor,
+                range_opts=range_opts,
+                is_expression=False,
+            ),
         )
 
     def encrypt_expression(
         self,
         expression: Mapping[str, Any],
         algorithm: str,
-        key_id: Optional[Binary] = None,
+        key_id: Optional[Union[Binary, uuid.UUID]] = None,
         key_alt_name: Optional[str] = None,
         query_type: Optional[str] = None,
         contention_factor: Optional[int] = None,
@@ -859,47 +865,49 @@ class ClientEncryption(Generic[_DocumentType]):
         Note that exactly one of ``key_id`` or  ``key_alt_name`` must be
         provided.
 
-        :Parameters:
-          - `expression`: The BSON aggregate or match expression to encrypt.
-          - `algorithm` (string): The encryption algorithm to use. See
+        :param expression: The BSON aggregate or match expression to encrypt.
+        :param algorithm` (string): The encryption algorithm to use. See
             :class:`Algorithm` for some valid options.
-          - `key_id`: Identifies a data key by ``_id`` which must be a
+        :param key_id: Identifies a data key by ``_id`` which must be a
             :class:`~bson.binary.Binary` with subtype 4 (
             :attr:`~bson.binary.UUID_SUBTYPE`).
-          - `key_alt_name`: Identifies a key vault document by 'keyAltName'.
-          - `query_type` (str): The query type to execute. See
+        :param key_alt_name: Identifies a key vault document by 'keyAltName'.
+        :param query_type` (str): The query type to execute. See
             :class:`QueryType` for valid options.
-          - `contention_factor` (int): The contention factor to use
+        :param contention_factor` (int): The contention factor to use
             when the algorithm is :attr:`Algorithm.INDEXED`.  An integer value
             *must* be given when the :attr:`Algorithm.INDEXED` algorithm is
             used.
-          - `range_opts`: Experimental only, not intended for public use.
+        :param range_opts: Experimental only, not intended for public use.
 
-        :Returns:
-          The encrypted expression, a :class:`~bson.RawBSONDocument`.
+        :return: The encrypted expression, a :class:`~bson.RawBSONDocument`.
+
+        .. versionchanged:: 4.7
+           ``key_id`` can now be passed in as a :class:`uuid.UUID`.
 
         .. versionadded:: 4.4
         """
-        return self._encrypt_helper(
-            value=expression,
-            algorithm=algorithm,
-            key_id=key_id,
-            key_alt_name=key_alt_name,
-            query_type=query_type,
-            contention_factor=contention_factor,
-            range_opts=range_opts,
-            is_expression=True,
+        return cast(
+            RawBSONDocument,
+            self._encrypt_helper(
+                value=expression,
+                algorithm=algorithm,
+                key_id=key_id,
+                key_alt_name=key_alt_name,
+                query_type=query_type,
+                contention_factor=contention_factor,
+                range_opts=range_opts,
+                is_expression=True,
+            ),
         )
 
     def decrypt(self, value: Binary) -> Any:
         """Decrypt an encrypted value.
 
-        :Parameters:
-          - `value` (Binary): The encrypted value, a
+        :param value` (Binary): The encrypted value, a
             :class:`~bson.binary.Binary` with subtype 6.
 
-        :Returns:
-          The decrypted BSON value.
+        :return: The decrypted BSON value.
         """
         self._check_closed()
         if not (isinstance(value, Binary) and value.subtype == 6):
@@ -913,13 +921,11 @@ class ClientEncryption(Generic[_DocumentType]):
     def get_key(self, id: Binary) -> Optional[RawBSONDocument]:
         """Get a data key by id.
 
-        :Parameters:
-          - `id` (Binary): The UUID of a key a which must be a
+        :param id` (Binary): The UUID of a key a which must be a
             :class:`~bson.binary.Binary` with subtype 4 (
             :attr:`~bson.binary.UUID_SUBTYPE`).
 
-        :Returns:
-          The key document.
+        :return: The key document.
 
         .. versionadded:: 4.2
         """
@@ -930,8 +936,7 @@ class ClientEncryption(Generic[_DocumentType]):
     def get_keys(self) -> Cursor[RawBSONDocument]:
         """Get all of the data keys.
 
-        :Returns:
-          An instance of :class:`~pymongo.cursor.Cursor` over the data key
+        :return: An instance of :class:`~pymongo.cursor.Cursor` over the data key
           documents.
 
         .. versionadded:: 4.2
@@ -943,13 +948,11 @@ class ClientEncryption(Generic[_DocumentType]):
     def delete_key(self, id: Binary) -> DeleteResult:
         """Delete a key document in the key vault collection that has the given ``key_id``.
 
-        :Parameters:
-          - `id` (Binary): The UUID of a key a which must be a
+        :param id` (Binary): The UUID of a key a which must be a
             :class:`~bson.binary.Binary` with subtype 4 (
             :attr:`~bson.binary.UUID_SUBTYPE`).
 
-        :Returns:
-          The delete result.
+        :return: The delete result.
 
         .. versionadded:: 4.2
         """
@@ -960,14 +963,12 @@ class ClientEncryption(Generic[_DocumentType]):
     def add_key_alt_name(self, id: Binary, key_alt_name: str) -> Any:
         """Add ``key_alt_name`` to the set of alternate names in the key document with UUID ``key_id``.
 
-        :Parameters:
-          - ``id``: The UUID of a key a which must be a
+        :param `id`: The UUID of a key a which must be a
             :class:`~bson.binary.Binary` with subtype 4 (
             :attr:`~bson.binary.UUID_SUBTYPE`).
-          - ``key_alt_name``: The key alternate name to add.
+        :param `key_alt_name`: The key alternate name to add.
 
-        :Returns:
-          The previous version of the key document.
+        :return: The previous version of the key document.
 
         .. versionadded:: 4.2
         """
@@ -979,11 +980,9 @@ class ClientEncryption(Generic[_DocumentType]):
     def get_key_by_alt_name(self, key_alt_name: str) -> Optional[RawBSONDocument]:
         """Get a key document in the key vault collection that has the given ``key_alt_name``.
 
-        :Parameters:
-          - `key_alt_name`: (str): The key alternate name of the key to get.
+        :param key_alt_name: (str): The key alternate name of the key to get.
 
-        :Returns:
-          The key document.
+        :return: The key document.
 
         .. versionadded:: 4.2
         """
@@ -996,14 +995,12 @@ class ClientEncryption(Generic[_DocumentType]):
 
         Also removes the ``keyAltNames`` field from the key document if it would otherwise be empty.
 
-        :Parameters:
-          - ``id``: The UUID of a key a which must be a
+        :param `id`: The UUID of a key a which must be a
             :class:`~bson.binary.Binary` with subtype 4 (
             :attr:`~bson.binary.UUID_SUBTYPE`).
-          - ``key_alt_name``: The key alternate name to remove.
+        :param `key_alt_name`: The key alternate name to remove.
 
-        :Returns:
-          Returns the previous version of the key document.
+        :return: Returns the previous version of the key document.
 
         .. versionadded:: 4.2
         """
@@ -1037,15 +1034,13 @@ class ClientEncryption(Generic[_DocumentType]):
     ) -> RewrapManyDataKeyResult:
         """Decrypts and encrypts all matching data keys in the key vault with a possibly new `master_key` value.
 
-        :Parameters:
-          - `filter`: A document used to filter the data keys.
-          - `provider`: The new KMS provider to use to encrypt the data keys,
+        :param filter: A document used to filter the data keys.
+        :param provider: The new KMS provider to use to encrypt the data keys,
             or ``None`` to use the current KMS provider(s).
-          - ``master_key``: The master key fields corresponding to the new KMS
+        :param `master_key`: The master key fields corresponding to the new KMS
             provider when ``provider`` is not ``None``.
 
-        :Returns:
-          A :class:`RewrapManyDataKeyResult`.
+        :return: A :class:`RewrapManyDataKeyResult`.
 
         This method allows you to re-encrypt all of your data-keys with a new CMK, or master key.
         Note that this does *not* require re-encrypting any of the data in your encrypted collections,
@@ -1089,7 +1084,7 @@ class ClientEncryption(Generic[_DocumentType]):
         result = self._key_vault_coll.bulk_write(replacements)
         return RewrapManyDataKeyResult(result)
 
-    def __enter__(self) -> "ClientEncryption":
+    def __enter__(self) -> ClientEncryption[_DocumentType]:
         return self
 
     def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
