@@ -19,14 +19,15 @@ from __future__ import annotations
 import atexit
 import time
 import weakref
-from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Tuple, cast
+from typing import TYPE_CHECKING, Any, Mapping, Optional, cast
 
 from pymongo import common, periodic_executor
 from pymongo._csot import MovingMinimum
-from pymongo.errors import NotPrimaryError, OperationFailure, _OperationCancelled
+from pymongo.errors import NetworkTimeout, NotPrimaryError, OperationFailure, _OperationCancelled
 from pymongo.hello import Hello
 from pymongo.lock import _create_lock
 from pymongo.periodic_executor import _shutdown_executors
+from pymongo.pool import _is_faas
 from pymongo.read_preferences import MovingAverage
 from pymongo.server_description import ServerDescription
 from pymongo.srv_resolver import _SrvResolver
@@ -51,6 +52,7 @@ class MonitorBase:
         The background thread is signaled to stop when the Topology or
         this instance is freed.
         """
+
         # We strongly reference the executor and it weakly references us via
         # this closure. When the monitor is freed, stop the executor soon.
         def target() -> bool:
@@ -138,7 +140,12 @@ class Monitor(MonitorBase):
             topology_settings,
             topology._create_pool_for_monitor(server_description.address),
         )
-        self.heartbeater = None
+        if topology_settings.server_monitoring_mode == "stream":
+            self._stream = True
+        elif topology_settings.server_monitoring_mode == "poll":
+            self._stream = False
+        else:
+            self._stream = not _is_faas()
 
     def cancel_check(self) -> None:
         """Cancel any concurrent hello check.
@@ -197,10 +204,12 @@ class Monitor(MonitorBase):
 
             # Update the Topology and clear the server pool on error.
             self._topology.on_change(
-                self._server_description, reset_pool=self._server_description.error
+                self._server_description,
+                reset_pool=self._server_description.error,
+                interrupt_connections=isinstance(self._server_description.error, NetworkTimeout),
             )
 
-            if (
+            if self._stream and (
                 self._server_description.is_server_type_known
                 and self._server_description.topology_version
             ):
@@ -237,7 +246,7 @@ class Monitor(MonitorBase):
             address = sd.address
             duration = time.monotonic() - start
             if self._publish:
-                awaited = bool(sd.is_server_type_known and sd.topology_version)
+                awaited = bool(self._stream and sd.is_server_type_known and sd.topology_version)
                 assert self._listeners is not None
                 self._listeners.publish_server_heartbeat_failed(address, duration, error, awaited)
             self._reset_connection()
@@ -255,7 +264,16 @@ class Monitor(MonitorBase):
         address = self._server_description.address
         if self._publish:
             assert self._listeners is not None
-            self._listeners.publish_server_heartbeat_started(address)
+            sd = self._server_description
+            # XXX: "awaited" could be incorrectly set to True in the rare case
+            # the pool checkout closes and recreates a connection.
+            awaited = bool(
+                self._pool.conns
+                and self._stream
+                and sd.is_server_type_known
+                and sd.topology_version
+            )
+            self._listeners.publish_server_heartbeat_started(address, awaited)
 
         if self._cancel_context and self._cancel_context.cancelled:
             self._reset_connection()
@@ -274,7 +292,7 @@ class Monitor(MonitorBase):
                 )
             return sd
 
-    def _check_with_socket(self, conn: Connection) -> Tuple[Hello, float]:
+    def _check_with_socket(self, conn: Connection) -> tuple[Hello, float]:
         """Return (Hello, round_trip_time).
 
         Can raise ConnectionFailure or OperationFailure.
@@ -284,7 +302,9 @@ class Monitor(MonitorBase):
         if conn.more_to_come:
             # Read the next streaming hello (MongoDB 4.4+).
             response = Hello(conn._next_reply(), awaitable=True)
-        elif conn.performed_handshake and self._server_description.topology_version:
+        elif (
+            self._stream and conn.performed_handshake and self._server_description.topology_version
+        ):
             # Initiate streaming hello (MongoDB 4.4+).
             response = conn._hello(
                 cluster_time,
@@ -315,8 +335,12 @@ class SrvMonitor(MonitorBase):
         self._seedlist = self._settings._seeds
         assert isinstance(self._settings.fqdn, str)
         self._fqdn: str = self._settings.fqdn
+        self._startup_time = time.monotonic()
 
     def _run(self) -> None:
+        # Don't poll right after creation, wait 60 seconds first
+        if time.monotonic() < self._startup_time + common.MIN_SRV_RESCAN_INTERVAL:
+            return
         seedlist = self._get_seedlist()
         if seedlist:
             self._seedlist = seedlist
@@ -326,7 +350,7 @@ class SrvMonitor(MonitorBase):
                 # Topology was garbage-collected.
                 self.close()
 
-    def _get_seedlist(self) -> Optional[List[Tuple[str, Any]]]:
+    def _get_seedlist(self) -> Optional[list[tuple[str, Any]]]:
         """Poll SRV records for a seedlist.
 
         Returns a list of ServerDescriptions.
@@ -383,7 +407,7 @@ class _RttMonitor(MonitorBase):
             self._moving_average.add_sample(sample)
             self._moving_min.add_sample(sample)
 
-    def get(self) -> Tuple[Optional[float], float]:
+    def get(self) -> tuple[Optional[float], float]:
         """Get the calculated average, or None if no samples yet and the min."""
         with self._lock:
             return self._moving_average.get(), self._moving_min.get()
