@@ -23,18 +23,15 @@ import sys as _sys
 import time as _time
 from errno import EINTR as _EINTR
 from ipaddress import ip_address as _ip_address
-from typing import TYPE_CHECKING, Any, Callable, List, Optional, TypeVar, Union
+from typing import TYPE_CHECKING, Any, Callable, Optional, TypeVar, Union
 
-from cryptography.x509 import load_der_x509_certificate as _load_der_x509_certificate
+import cryptography.x509 as x509
+import service_identity
 from OpenSSL import SSL as _SSL
 from OpenSSL import crypto as _crypto
-from service_identity import CertificateError as _SICertificateError
-from service_identity import VerificationError as _SIVerificationError
-from service_identity.pyopenssl import verify_hostname as _verify_hostname
-from service_identity.pyopenssl import verify_ip_address as _verify_ip_address
 
 from pymongo.errors import ConfigurationError as _ConfigurationError
-from pymongo.errors import _CertificateError
+from pymongo.errors import _CertificateError  # type:ignore[attr-defined]
 from pymongo.ocsp_cache import _OCSPCache
 from pymongo.ocsp_support import _load_trusted_ca_certs, _ocsp_callback
 from pymongo.socket_checker import SocketChecker as _SocketChecker
@@ -44,7 +41,6 @@ from pymongo.write_concern import validate_boolean
 if TYPE_CHECKING:
     from ssl import VerifyMode
 
-    from cryptography.x509 import Certificate
 
 _T = TypeVar("_T")
 
@@ -86,7 +82,7 @@ def _is_ip_address(address: Any) -> bool:
     try:
         _ip_address(address)
         return True
-    except (ValueError, UnicodeError):  # noqa: B014
+    except (ValueError, UnicodeError):
         return False
 
 
@@ -122,8 +118,8 @@ class _sslConn(_SSL.Connection):
                 # Check for closed socket.
                 if self.fileno() == -1:
                     if timeout and _time.monotonic() - start > timeout:
-                        raise _socket.timeout("timed out")
-                    raise SSLError("Underlying socket has been closed")
+                        raise _socket.timeout("timed out") from None
+                    raise SSLError("Underlying socket has been closed") from None
                 if isinstance(exc, _SSL.WantReadError):
                     want_read = True
                     want_write = False
@@ -135,7 +131,7 @@ class _sslConn(_SSL.Connection):
                     want_write = True
                 self.socket_checker.select(self, want_read, want_write, timeout)
                 if timeout and _time.monotonic() - start > timeout:
-                    raise _socket.timeout("timed out")
+                    raise _socket.timeout("timed out") from None
                 continue
 
     def do_handshake(self, *args: Any, **kwargs: Any) -> None:
@@ -169,7 +165,7 @@ class _sslConn(_SSL.Connection):
             # XXX: It's not clear if this can actually happen. PyOpenSSL
             # doesn't appear to have any interrupt handling, nor any interrupt
             # errors for OpenSSL connections.
-            except OSError as exc:  # noqa: B014
+            except OSError as exc:
                 if _errno_from_exception(exc) == _EINTR:
                     continue
                 raise
@@ -184,7 +180,7 @@ class _CallbackData:
     """Data class which is passed to the OCSP callback."""
 
     def __init__(self) -> None:
-        self.trusted_ca_certs: Optional[List[Certificate]] = None
+        self.trusted_ca_certs: Optional[list[x509.Certificate]] = None
         self.check_ocsp_endpoint: Optional[bool] = None
         self.ocsp_response_cache = _OCSPCache()
 
@@ -226,10 +222,10 @@ class SSLContext:
         """Setter for verify_mode."""
 
         def _cb(
-            connobj: _SSL.Connection,
-            x509obj: _crypto.X509,
-            errnum: int,
-            errdepth: int,
+            _connobj: _SSL.Connection,
+            _x509obj: _crypto.X509,
+            _errnum: int,
+            _errdepth: int,
             retcode: int,
         ) -> bool:
             # It seems we don't need to do anything here. Twisted doesn't,
@@ -270,7 +266,7 @@ class SSLContext:
         return self._ctx.set_options(0)
 
     def __set_options(self, value: int) -> None:
-        # Explcitly convert to int, since newer CPython versions
+        # Explicitly convert to int, since newer CPython versions
         # use enum.IntFlag for options. The values are the same
         # regardless of implementation.
         self._ctx.set_options(int(value))
@@ -295,7 +291,7 @@ class SSLContext:
         # Password callback MUST be set first or it will be ignored.
         if password:
 
-            def _pwcb(max_length: int, prompt_twice: bool, user_data: bytes) -> bytes:
+            def _pwcb(_max_length: int, _prompt_twice: bool, _user_data: bytes) -> bytes:
                 # XXX:We could check the password length against what OpenSSL
                 # tells us is the max, but we can't raise an exception, so...
                 # warn?
@@ -336,11 +332,12 @@ class SSLContext:
         """Attempt to load CA certs from Windows trust store."""
         cert_store = self._ctx.get_cert_store()
         oid = _stdlibssl.Purpose.SERVER_AUTH.oid
+
         for cert, encoding, trust in _stdlibssl.enum_certificates(store):  # type: ignore
             if encoding == "x509_asn":
                 if trust is True or oid in trust:
                     cert_store.add_cert(
-                        _crypto.X509.from_cryptography(_load_der_x509_certificate(cert))
+                        _crypto.X509.from_cryptography(x509.load_der_x509_certificate(cert))
                     )
 
     def load_default_certs(self) -> None:
@@ -404,11 +401,16 @@ class SSLContext:
             # XXX: Do this in a callback registered with
             # SSLContext.set_info_callback? See Twisted for an example.
             if self.check_hostname and server_hostname is not None:
+                from service_identity import pyopenssl
+
                 try:
                     if _is_ip_address(server_hostname):
-                        _verify_ip_address(ssl_conn, server_hostname)
+                        pyopenssl.verify_ip_address(ssl_conn, server_hostname)
                     else:
-                        _verify_hostname(ssl_conn, server_hostname)
-                except (_SICertificateError, _SIVerificationError) as exc:
-                    raise _CertificateError(str(exc))
+                        pyopenssl.verify_hostname(ssl_conn, server_hostname)
+                except (  # type:ignore[misc]
+                    service_identity.SICertificateError,
+                    service_identity.SIVerificationError,
+                ) as exc:
+                    raise _CertificateError(str(exc)) from None
         return ssl_conn
