@@ -13,10 +13,11 @@
 # limitations under the License.
 
 """Represent one server the driver is connected to."""
+from __future__ import annotations
 
 import time
 import warnings
-from typing import Any, Dict, Mapping, Optional, Set, Tuple
+from typing import Any, Mapping, Optional
 
 from bson import EPOCH_NAIVE
 from bson.objectid import ObjectId
@@ -28,12 +29,11 @@ from pymongo.typings import ClusterTime, _Address
 class ServerDescription:
     """Immutable representation of one server.
 
-    :Parameters:
-      - `address`: A (host, port) pair
-      - `hello`: Optional Hello instance
-      - `round_trip_time`: Optional float
-      - `error`: Optional, the last error attempting to connect to the server
-      - `round_trip_time`: Optional float, the min latency from the most recent samples
+    :param address: A (host, port) pair
+    :param hello: Optional Hello instance
+    :param round_trip_time: Optional float
+    :param error: Optional, the last error attempting to connect to the server
+    :param round_trip_time: Optional float, the min latency from the most recent samples
     """
 
     __slots__ = (
@@ -129,7 +129,7 @@ class ServerDescription:
         return SERVER_TYPE._fields[self._server_type]
 
     @property
-    def all_hosts(self) -> Set[Tuple[str, int]]:
+    def all_hosts(self) -> set[tuple[str, int]]:
         """List of hosts, passives, and arbiters known to this server."""
         return self._all_hosts
 
@@ -143,7 +143,7 @@ class ServerDescription:
         return self._replica_set_name
 
     @property
-    def primary(self) -> Optional[Tuple[str, int]]:
+    def primary(self) -> Optional[tuple[str, int]]:
         """This server's opinion about who the primary is, or None."""
         return self._primary
 
@@ -180,7 +180,7 @@ class ServerDescription:
         return self._cluster_time
 
     @property
-    def election_tuple(self) -> Tuple[Optional[int], Optional[ObjectId]]:
+    def election_tuple(self) -> tuple[Optional[int], Optional[ObjectId]]:
         warnings.warn(
             "'election_tuple' is deprecated, use  'set_version' and 'election_id' instead",
             DeprecationWarning,
@@ -189,7 +189,7 @@ class ServerDescription:
         return self._set_version, self._election_id
 
     @property
-    def me(self) -> Optional[Tuple[str, int]]:
+    def me(self) -> Optional[tuple[str, int]]:
         return self._me
 
     @property
@@ -243,8 +243,7 @@ class ServerDescription:
     def retryable_writes_supported(self) -> bool:
         """Checks if this server supports retryable writes."""
         return (
-            self._ls_timeout_minutes is not None
-            and self._server_type in (SERVER_TYPE.Mongos, SERVER_TYPE.RSPrimary)
+            self._server_type in (SERVER_TYPE.Mongos, SERVER_TYPE.RSPrimary)
         ) or self._server_type == SERVER_TYPE.LoadBalancer
 
     @property
@@ -256,7 +255,7 @@ class ServerDescription:
     def topology_version(self) -> Optional[Mapping[str, Any]]:
         return self._topology_version
 
-    def to_unknown(self, error: Optional[Exception] = None) -> "ServerDescription":
+    def to_unknown(self, error: Optional[Exception] = None) -> ServerDescription:
         unknown = ServerDescription(self.address, error=error)
         unknown._topology_version = self.topology_version
         return unknown
@@ -297,4 +296,4 @@ class ServerDescription:
         )
 
     # For unittesting only. Use under no circumstances!
-    _host_to_round_trip_time: Dict = {}
+    _host_to_round_trip_time: dict = {}
